@@ -8,7 +8,7 @@ from datetime import datetime
 from functools import partial
 import importlib.resources
 import os
-from types import SimpleNamespace
+from pathlib import Path
 import typing
 from typing import Any, TypeVar
 
@@ -23,23 +23,26 @@ import click
 from loguru import logger
 from pydantic import BaseModel, ValidationError
 import sqlalchemy as sa
+import toga
 from typing_extensions import Concatenate, Literal, ParamSpec, TypeAlias, TypedDict
 from yarl import URL
 
 from instawow import __version__, db, matchers, models
 from instawow import results as R
-from instawow.common import Strategy
-from instawow.config import Config
-from instawow.manager import Manager, TraceRequestCtx, init_web_client, is_outdated
-from instawow.resolvers import CatalogueEntry, Defn
+from instawow.cataloguer import CatalogueEntry
+from instawow.common import Flavour, Strategy
+from instawow.config import Config, GlobalConfig
+from instawow.manager import LocksType, Manager, TraceRequestCtx, init_web_client, is_outdated
+from instawow.resolvers import Defn
+from instawow.utils import evolve_model_obj
 from instawow.utils import run_in_thread as t
 from instawow.utils import uniq
 
-from . import frontend, templates
+from . import frontend
 
 _T = TypeVar('_T')
 _P = ParamSpec('_P')
-_ManagerWorkQueueItem: TypeAlias = 'tuple[asyncio.Future[Any], str, Callable[..., Awaitable[Any]]]'
+ManagerBoundCoroFn: TypeAlias = 'Callable[Concatenate[Manager, _P], Awaitable[_T]]'
 
 
 LOCALHOST = '127.0.0.1'
@@ -59,10 +62,16 @@ def _reraise_validation_error(
         yield
     except ValidationError as error:
         errors = error.errors()
-        logger.info(f'received invalid request: {(values, errors)}')
+        logger.info(f'invalid request: {(values, errors)}')
         raise error_class(data=errors) from error
 
 
+@t
+def _read_config(profile: str) -> Config:
+    with _reraise_validation_error(_ConfigError):
+        return Config.read(None, profile)
+
+
 methods: list[tuple[str, type[BaseParams]]] = []
 
 
@@ -76,15 +85,19 @@ def _register_method(method: str):
 
 class BaseParams(BaseModel):
     @classmethod
-    def bind(cls, method: str, managers: _ManagerWorkQueue) -> JsonRpcMethod:
+    def bind(
+        cls, method: str, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> JsonRpcMethod:
         async def respond(**kwargs: Any) -> BaseModel:
             with _reraise_validation_error(InvalidParamsError, kwargs):
                 params = cls.parse_obj(kwargs)
-            return await params.respond(managers)
+            return await params.respond(managers, app_window)
 
         return JsonRpcMethod(respond, name=method)
 
-    async def respond(self, managers: _ManagerWorkQueue) -> Any:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> Any:
         raise NotImplementedError
 
 
@@ -97,35 +110,51 @@ class _DefnParamMixin(BaseModel):
 
 
 @_register_method('config/write')
-class WriteConfigParams(BaseParams):
-    values: typing.Dict[str, Any]
+class WriteConfigParams(_ProfileParamMixin, BaseParams):
+    addon_dir: Path
+    game_flavour: Flavour
     infer_game_flavour: bool
 
-    @t
-    def respond(self, managers: _ManagerWorkQueue) -> Config:
-        with _reraise_validation_error(_ConfigError):
-            config = Config(**self.values)
-            if self.infer_game_flavour:
-                config.game_flavour = Config.infer_flavour(config.addon_dir)
-            config.write()
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> Config:
+        async with managers.locks['modify profile', self.profile]:
+            with _reraise_validation_error(_ConfigError):
+                config = Config(
+                    global_config=await t(GlobalConfig.read)(),
+                    profile=self.profile,
+                    addon_dir=self.addon_dir,
+                    game_flavour=self.game_flavour,
+                )
+                if self.infer_game_flavour:
+                    values = {
+                        **config.dict(),
+                        'game_flavour': Config.infer_flavour(config.addon_dir),
+                    }
+                    config = Config(**values)
 
-        # Dispose of the ``Manager`` corresponding to the profile if any
-        # so that the configuration is reloaded on next invocation
-        managers.unload(config.profile)
-        return config
+                await t(config.write)()
+
+            # Unload the ``Manager`` instance corresponding to this profile for
+            # the config to be reloaded on the next request
+            managers.unload(config.profile)
+
+            return config
 
 
 @_register_method('config/read')
 class ReadConfigParams(_ProfileParamMixin, BaseParams):
-    @t
-    def respond(self, managers: _ManagerWorkQueue) -> Config:
-        with _reraise_validation_error(_ConfigError):
-            return Config.read(self.profile)
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> Config:
+        return await _read_config(self.profile)
 
 
 @_register_method('config/delete')
 class DeleteConfigParams(_ProfileParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> None:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> None:
         async def delete_profile(manager: Manager):
             await t(manager.config.delete)()
             managers.unload(self.profile)
@@ -136,8 +165,10 @@ class DeleteConfigParams(_ProfileParamMixin, BaseParams):
 @_register_method('config/list')
 class ListProfilesParams(BaseParams):
     @t
-    def respond(self, managers: _ManagerWorkQueue) -> list[str]:
-        return Config.list_profiles()
+    def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[str]:
+        return GlobalConfig().list_profiles()
 
 
 class Source(TypedDict):
@@ -150,8 +181,10 @@ class Source(TypedDict):
 
 @_register_method('sources/list')
 class ListSourcesParams(_ProfileParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[Source]:
-        manager = await managers.run(self.profile, _get_manager)
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[Source]:
+        manager = await managers.get_manager(self.profile)
         return [
             {
                 'source': r.source,
@@ -166,8 +199,10 @@ class ListSourcesParams(_ProfileParamMixin, BaseParams):
 
 @_register_method('list')
 class ListInstalledParams(_ProfileParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[models.Pkg]:
-        manager = await managers.run(self.profile, _get_manager)
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[models.Pkg]:
+        manager = await managers.get_manager(self.profile)
         installed_pkgs = (
             manager.database.execute(sa.select(db.pkg).order_by(sa.func.lower(db.pkg.c.name)))
             .mappings()
@@ -176,6 +211,30 @@ class ListInstalledParams(_ProfileParamMixin, BaseParams):
         return [models.Pkg.from_row_mapping(manager.database, p) for p in installed_pkgs]
 
 
+@_register_method('search')
+class SearchParams(_ProfileParamMixin, BaseParams):
+    search_terms: str
+    limit: int
+    sources: typing.Set[str]
+    start_date: typing.Optional[datetime]
+    installed_only: bool
+
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[CatalogueEntry]:
+        return await managers.run(
+            self.profile,
+            partial(
+                Manager.search,
+                search_terms=self.search_terms,
+                limit=self.limit,
+                sources=self.sources,
+                start_date=self.start_date,
+                installed_only=self.installed_only,
+            ),
+        )
+
+
 class SuccessResult(TypedDict):
     status: Literal['success']
     addon: models.Pkg
@@ -186,26 +245,6 @@ class ErrorResult(TypedDict):
     message: str
 
 
-@_register_method('search')
-class SearchParams(_ProfileParamMixin, BaseParams):
-    search_terms: str
-    limit: int
-    sources: typing.Set[str]
-    start_date: typing.Optional[datetime]
-
-    async def respond(self, managers: _ManagerWorkQueue) -> list[CatalogueEntry]:
-        return await managers.run(
-            self.profile,
-            partial(
-                Manager.search,
-                search_terms=self.search_terms,
-                limit=self.limit,
-                sources=self.sources,
-                start_date=self.start_date,
-            ),
-        )
-
-
 @_register_method('resolve')
 class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
     async def _resolve(self, manager: Manager) -> dict[Defn, Any]:
@@ -214,16 +253,18 @@ class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
                 pair = manager.pair_uri(defn.alias)
                 if pair:
                     source, alias = pair
-                    defn = defn.with_(source=source, alias=alias)
+                    defn = evolve_model_obj(defn, source=source, alias=alias)
             return defn
 
         return await manager.resolve(list(map(extract_source, self.defns)))
 
-    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(self.profile, self._resolve)
         return [
             {'status': 'success', 'addon': r}
-            if models.is_pkg(r)
+            if isinstance(r, models.Pkg)
             else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
@@ -233,7 +274,9 @@ class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
 class InstallParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
     replace: bool
 
-    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(
             self.profile, partial(Manager.install, defns=self.defns, replace=self.replace)
         )
@@ -247,7 +290,9 @@ class InstallParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
 
 @_register_method('update')
 class UpdateParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(
             self.profile, partial(Manager.update, defns=self.defns, retain_defn_strategy=True)
         )
@@ -263,7 +308,9 @@ class UpdateParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
 class RemoveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
     keep_folders: bool
 
-    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(
             self.profile, partial(Manager.remove, defns=self.defns, keep_folders=self.keep_folders)
         )
@@ -277,7 +324,9 @@ class RemoveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
 
 @_register_method('pin')
 class PinParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(self.profile, partial(Manager.pin, defns=self.defns))
         return [
             {'status': r.status, 'addon': r.pkg}
@@ -291,23 +340,20 @@ class PinParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
 class GetChangelogParams(_ProfileParamMixin, BaseParams):
     changelog_url: str
 
-    async def respond(self, managers: _ManagerWorkQueue) -> str:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> str:
         return await managers.run(
             self.profile, partial(Manager.get_changelog, uri=self.changelog_url)
         )
 
 
-class ReconcileResult(TypedDict):
-    reconciled: list[ReconcileResult_AddonMatch]
-    unreconciled: list[ReconcileResult_AddonMatch]
-
-
-class ReconcileResult_AddonMatch(TypedDict):
-    folders: list[ReconcileResult_AddonFolder]
+class AddonMatch(TypedDict):
+    folders: list[AddonMatch_AddonFolder]
     matches: list[models.Pkg]
 
 
-class ReconcileResult_AddonFolder(TypedDict):
+class AddonMatch_AddonFolder(TypedDict):
     name: str
     version: str
 
@@ -316,31 +362,64 @@ class ReconcileResult_AddonFolder(TypedDict):
 class ReconcileParams(_ProfileParamMixin, BaseParams):
     matcher: Literal['toc_source_ids', 'folder_name_subsets', 'addon_names_with_folder_names']
 
-    async def respond(self, managers: _ManagerWorkQueue) -> ReconcileResult:
-        leftovers = await managers.run(self.profile, t(matchers.get_unreconciled_folder_set))
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[AddonMatch]:
+        leftovers = await managers.run(self.profile, t(matchers.get_unreconciled_folders))
         match_groups: matchers.FolderAndDefnPairs = await managers.run(
             self.profile, partial(getattr(matchers, f'match_{self.matcher}'), leftovers=leftovers)
         )
-        uniq_defns = uniq(d for _, b in match_groups for d in b)
         resolve_results = await managers.run(
-            self.profile, partial(Manager.resolve, defns=uniq_defns)
+            self.profile,
+            partial(Manager.resolve, defns=uniq(d for _, b in match_groups for d in b)),
         )
-        return {
-            'reconciled': [
-                {
-                    'folders': [{'name': f.name, 'version': f.version} for f in a],
-                    'matches': [r for i in d for r in (resolve_results[i],) if models.is_pkg(r)],
-                }
-                for a, d in match_groups
-            ],
-            'unreconciled': [
-                {
-                    'folders': [{'name': f.name, 'version': f.version}],
-                    'matches': [],
-                }
-                for f in sorted(leftovers - frozenset(i for a, _ in match_groups for i in a))
-            ],
-        }
+        matches = [
+            (a, m)
+            for a, s in match_groups
+            for m in ([r for d in s for r in (resolve_results[d],) if isinstance(r, models.Pkg)],)
+            if m
+        ]
+        return [
+            AddonMatch(folders=[{'name': f.name, 'version': f.version} for f in a], matches=m)
+            for a, m in matches
+        ] + [
+            AddonMatch(folders=[{'name': f.name, 'version': f.version}], matches=[])
+            for f in sorted(leftovers - frozenset(i for a, _ in matches for i in a))
+        ]
+
+
+class ReconcileInstalledCandidate(TypedDict):
+    installed_addon: models.Pkg
+    alternative_addons: list[models.Pkg]
+
+
+@_register_method('get_reconcile_installed_candidates')
+class GetReconcileInstalledCandidatesParams(_ProfileParamMixin, BaseParams):
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[ReconcileInstalledCandidate]:
+        manager = await managers.get_manager(self.profile)
+        installed_pkgs = (
+            models.Pkg.from_row_mapping(manager.database, p)
+            for p in manager.database.execute(
+                sa.select(db.pkg).order_by(sa.func.lower(db.pkg.c.name))
+            )
+            .mappings()
+            .all()
+        )
+        defn_groups = await managers.run(
+            self.profile, partial(Manager.find_equivalent_pkg_defns, pkgs=installed_pkgs)
+        )
+        resolve_results = await managers.run(
+            self.profile,
+            partial(Manager.resolve, defns=uniq(d for b in defn_groups.values() for d in b)),
+        )
+        return [
+            {'installed_addon': p, 'alternative_addons': m}
+            for p, s in defn_groups.items()
+            for m in ([r for d in s for r in (resolve_results[d],) if isinstance(r, models.Pkg)],)
+            if m
+        ]
 
 
 class DownloadProgressReport(TypedDict):
@@ -350,10 +429,12 @@ class DownloadProgressReport(TypedDict):
 
 @_register_method('get_download_progress')
 class GetDownloadProgressParams(_ProfileParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[DownloadProgressReport]:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> list[DownloadProgressReport]:
         return [
             {'defn': Defn.from_pkg(p), 'progress': r}
-            for p, r in await managers.get_download_progress(self.profile)
+            for p, r in await managers.get_manager_download_progress(self.profile)
         ]
 
 
@@ -364,7 +445,9 @@ class GetVersionResult(TypedDict):
 
 @_register_method('meta/get_version')
 class GetVersionParams(BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> GetVersionResult:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> GetVersionResult:
         outdated, new_version = await is_outdated()
         return {
             'installed_version': __version__,
@@ -376,7 +459,9 @@ class GetVersionParams(BaseParams):
 class OpenUrlParams(BaseParams):
     url: str
 
-    async def respond(self, managers: _ManagerWorkQueue) -> None:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> None:
         click.launch(self.url)
 
 
@@ -384,7 +469,9 @@ class OpenUrlParams(BaseParams):
 class RevealFolderParams(BaseParams):
     path_parts: typing.List[str]
 
-    async def respond(self, managers: _ManagerWorkQueue) -> None:
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> None:
         click.launch(os.path.join(*self.path_parts), locate=True)
 
 
@@ -396,13 +483,13 @@ class SelectFolderResult(TypedDict):
 class SelectFolderParams(BaseParams):
     initial_folder: typing.Optional[str]
 
-    async def respond(self, managers: _ManagerWorkQueue) -> SelectFolderResult:
-        from .app import InstawowApp
-
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> SelectFolderResult:
+        if not app_window:
+            raise RuntimeError('No app to bind to')
         try:
-            (selection,) = InstawowApp.app.iw_window.select_folder_dialog(
-                'Select folder', self.initial_folder
-            )
+            (selection,) = app_window.select_folder_dialog('Select folder', self.initial_folder)
         except ValueError:
             selection = None
         return {'selection': selection}
@@ -417,10 +504,12 @@ class ConfirmDialogueParams(BaseParams):
     title: str
     message: str
 
-    async def respond(self, managers: _ManagerWorkQueue) -> ConfirmDialogueResult:
-        from .app import InstawowApp
-
-        ok = InstawowApp.app.iw_window.confirm_dialog(self.title, self.message)
+    async def respond(
+        self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
+    ) -> ConfirmDialogueResult:
+        if not app_window:
+            raise RuntimeError('No app to bind to')
+        ok = app_window.confirm_dialog(self.title, self.message)
         return {'ok': ok}
 
 
@@ -429,7 +518,7 @@ def _init_json_rpc_web_client(
 ) -> aiohttp.ClientSession:
     async def do_on_request_end(
         client_session: aiohttp.ClientSession,
-        trace_config_ctx: SimpleNamespace,
+        trace_config_ctx: Any,
         params: aiohttp.TraceRequestEndParams,
     ) -> None:
         trace_request_ctx: TraceRequestCtx = trace_config_ctx.trace_request_ctx
@@ -444,7 +533,7 @@ def _init_json_rpc_web_client(
             entry = (
                 trace_request_ctx['manager'],
                 trace_request_ctx['pkg'],
-                lambda: content.total_bytes / content_length,  # type: ignore
+                lambda: content.total_bytes / content_length,
             )
             progress_reporters.add(entry)
             content.on_eof(lambda: progress_reporters.remove(entry))
@@ -455,69 +544,74 @@ def _init_json_rpc_web_client(
     return init_web_client(trace_configs=[trace_config])
 
 
-async def _get_manager(manager: Manager):
-    return manager
-
-
 class _ManagerWorkQueue:
-    def __init__(self) -> None:
+    def __init__(self):
         self._loop = asyncio.get_running_loop()
-        self._managers: dict[str, Manager] = {}
-        self._queue: asyncio.Queue[_ManagerWorkQueueItem] = asyncio.Queue()
-        self._progress_reporters: set[tuple[Manager, models.Pkg, Callable[[], float]]] = set()
-        self._web_client = _init_json_rpc_web_client(self._progress_reporters)
-        self._locks: defaultdict[str, asyncio.Lock] = defaultdict(asyncio.Lock)
 
-    async def cleanup(self) -> None:
-        for manager in self._managers.values():
-            manager.database.close()
+        self._queue: asyncio.Queue[
+            tuple[asyncio.Future[object], str, ManagerBoundCoroFn[..., object]]
+        ] = asyncio.Queue()
+        self._managers: dict[str, tuple[Manager, Callable[[], None]]] = {}
+        self._progress_reporters: set[tuple[Manager, models.Pkg, Callable[[], float]]] = set()
+
+        self._web_client = _init_json_rpc_web_client(self._progress_reporters)
+        self.locks: LocksType = defaultdict(asyncio.Lock)
+
+    async def cleanup(self):
+        for _, close_db_conn in self._managers.values():
+            close_db_conn()
         await self._web_client.close()
 
-    def unload(self, profile: str) -> None:
-        manager = self._managers.pop(profile, None)
-        if manager:
-            manager.database.close()
+    def unload(self, profile: str):
+        if profile in self._managers:
+            _, close_db_conn = self._managers.pop(profile)
+            close_db_conn()
 
-    async def listen(self) -> None:
-        async def schedule(
-            future: asyncio.Future[Any], profile: str, coro_fn: Callable[..., Awaitable[Any]]
-        ):
-            try:
-                async with self._locks[f"load profile '{profile}'"]:
-                    try:
-                        manager = self._managers[profile]
-                    except KeyError:
-                        with _reraise_validation_error(_ConfigError):
-                            config = await t(Config.read)(profile)
+    async def _schedule_item(
+        self,
+        future: asyncio.Future[object],
+        profile: str,
+        coro_fn: ManagerBoundCoroFn[_P, object],
+    ):
+        try:
+            async with self.locks['modify profile', profile]:
+                try:
+                    manager, _ = self._managers[profile]
+                except KeyError:
+                    config = await _read_config(profile)
+                    self._managers[profile] = Manager.from_config(config)
+                    manager, _ = self._managers[profile]
 
-                        manager = self._managers[profile] = Manager.from_config(config)
-
-                result = await coro_fn(manager)
-            except BaseException as exc:
-                future.set_exception(exc)
-            else:
-                future.set_result(result)
-
-        Manager.contextualise(web_client=self._web_client, locks=self._locks)
+            result = await coro_fn(manager)
+        except BaseException as exc:
+            future.set_exception(exc)
+        else:
+            future.set_result(result)
 
+    async def listen(self):
+        Manager.contextualise(web_client=self._web_client, locks=self.locks)
         while True:
             item = await self._queue.get()
-            asyncio.create_task(schedule(*item))
+            asyncio.create_task(self._schedule_item(*item))
             self._queue.task_done()
 
-    async def run(
-        self, profile: str, coro_fn: Callable[Concatenate[Manager, _P], Awaitable[_T]]
-    ) -> _T:
+    async def run(self, profile: str, coro_fn: ManagerBoundCoroFn[..., _T]) -> _T:
         future = self._loop.create_future()
         self._queue.put_nowait((future, profile, coro_fn))
         return await asyncio.wait_for(future, None)
 
-    async def get_download_progress(self, profile: str) -> Iterator[tuple[models.Pkg, float]]:
-        manager = await self.run(profile, _get_manager)
+    async def get_manager(self, profile: str):
+        async def get_manager(manager: Manager):
+            return manager
+
+        return await self.run(profile, get_manager)
+
+    async def get_manager_download_progress(self, profile: str):
+        manager = await self.get_manager(profile)
         return ((p, r()) for m, p, r in self._progress_reporters if m is manager)
 
 
-async def create_app() -> aiohttp.web.Application:
+async def create_app(app_window: toga.MainWindow | None = None):
     managers = _ManagerWorkQueue()
 
     async def start_managers(app: aiohttp.web.Application):
@@ -529,7 +623,7 @@ async def create_app() -> aiohttp.web.Application:
     async def get_index(request: aiohttp.web.Request):
         return aiohttp.web.Response(
             content_type='text/html',
-            text=importlib.resources.read_text(templates, 'index.html'),
+            text=importlib.resources.read_text(frontend, 'index.html'),
         )
 
     async def get_static_file(request: aiohttp.web.Request):
@@ -546,16 +640,39 @@ async def create_app() -> aiohttp.web.Application:
             text=importlib.resources.read_text(frontend, filename),
         )
 
-    def _json_serialize(value: dict[str, Any]) -> str:
+    def json_serialize(value: dict[str, Any]):
         return BaseModel.construct(**value).json()
 
     rpc_server = WsJsonRpcServer(
-        json_serialize=_json_serialize,
+        json_serialize=json_serialize,
         middlewares=rpc_middlewares.DEFAULT_MIDDLEWARES,
     )
-    rpc_server.add_methods([m.bind(n, managers) for n, m in methods])
+    rpc_server.add_methods([m.bind(n, managers, app_window) for n, m in methods])
 
-    app = aiohttp.web.Application()
+    @aiohttp.web.middleware
+    async def enforce_same_origin(
+        request: aiohttp.web.Request,
+        handler: Callable[[aiohttp.web.Request], Awaitable[aiohttp.web.StreamResponse]],
+    ):
+        if request.remote != LOCALHOST:
+            raise aiohttp.web.HTTPUnauthorized
+
+        if request.path.startswith('/api'):
+            origin = request.headers.get(aiohttp.hdrs.ORIGIN)
+            if origin is None:
+                raise aiohttp.web.HTTPUnauthorized
+
+            origin_url = URL(origin)
+            if (
+                origin_url.scheme != request.url.scheme
+                or origin_url.host != request.url.host
+                or origin_url.port != request.url.port
+            ):
+                raise aiohttp.web.HTTPUnauthorized
+
+        return await handler(request)
+
+    app = aiohttp.web.Application(middlewares=[enforce_same_origin])
     app.add_routes(
         [
             aiohttp.web.get('/', get_index),
@@ -570,9 +687,8 @@ async def create_app() -> aiohttp.web.Application:
     return app
 
 
-async def prepare() -> tuple[URL, Callable[[], Awaitable[None]]]:
+async def run_app(app: aiohttp.web.Application):
     "Fire up the server."
-    app = await create_app()
     app_runner = aiohttp.web.AppRunner(app)
     await app_runner.setup()
     assert app_runner.server  # Server is created during setup
