@@ -1,28 +1,38 @@
+# pyright: reportUnknownMemberType=false
+
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Awaitable, Callable, Generator, Iterable, Iterator, Sequence, Set
+from collections.abc import (
+    Awaitable,
+    Callable,
+    Collection,
+    Generator,
+    Iterable,
+    Iterator,
+    Sequence,
+    Set,
+)
 from contextlib import contextmanager
 from datetime import datetime, timezone
-from enum import Enum
 from functools import partial, wraps
-import importlib.util
 from itertools import chain, repeat
 from pathlib import Path
 import textwrap
 from typing import Any, NoReturn, TypeVar, overload
 
 import click
+from loguru import logger
 
 from . import __version__, _deferred_types, db
 from . import manager as _manager
 from . import models
 from . import results as R
-from .common import Strategy
-from .config import Config, Flavour, setup_logging
+from .common import Flavour, Strategy
+from .config import Config, GlobalConfig, setup_logging
 from .plugins import load_plugins
 from .resolvers import ChangelogFormat, Defn
-from .utils import StrEnum, cached_property, gather, tabulate, uniq
+from .utils import StrEnum, all_eq, cached_property, evolve_model_obj, gather, tabulate, uniq
 
 _T = TypeVar('_T')
 _F = TypeVar('_F', bound='Callable[..., object]')
@@ -67,7 +77,7 @@ class Report:
 
     def generate(self) -> None:
         manager_wrapper: ManagerWrapper | None = click.get_current_context().obj
-        if manager_wrapper and manager_wrapper.manager.config.auto_update_check:
+        if manager_wrapper and manager_wrapper.manager.config.global_config.auto_update_check:
             outdated, new_version = run_with_progress(_manager.is_outdated())
             if outdated:
                 click.echo(f'{self.WARNING_SYMBOL} instawow-{new_version} is available')
@@ -132,20 +142,21 @@ def _init_cli_web_client(
 
 
 @contextmanager
-def _cancel_tickers() -> Iterator[set[asyncio.Task[None]]]:
+def _cancel_tickers(progress_bar: _deferred_types.prompt_toolkit.shortcuts.ProgressBar):
     tickers: set[asyncio.Task[None]] = set()
     try:
         yield tickers
     finally:
         for ticker in tickers:
             ticker.cancel()
+        progress_bar.invalidate()
 
 
 def run_with_progress(awaitable: Awaitable[_T]) -> _T:
     from .prompts import make_progress_bar
 
     async def run():
-        with make_progress_bar() as progress_bar, _cancel_tickers() as tickers:
+        with make_progress_bar() as progress_bar, _cancel_tickers(progress_bar) as tickers:
             async with _init_cli_web_client(progress_bar, tickers) as web_client:
                 _manager.Manager.contextualise(web_client=web_client)
                 return await awaitable
@@ -153,7 +164,7 @@ def run_with_progress(awaitable: Awaitable[_T]) -> _T:
     return asyncio.run(run())
 
 
-def _override_asyncio_loop_policy() -> None:
+def _override_asyncio_loop_policy():
     # The proactor event loop which became the default loop on Windows
     # in Python 3.8 is causing issues with aiohttp.
     # See https://github.com/aio-libs/aiohttp/issues/4324
@@ -162,7 +173,7 @@ def _override_asyncio_loop_policy() -> None:
         asyncio.set_event_loop_policy(policy())
 
 
-def _apply_patches() -> None:
+def _apply_patches():
     _override_asyncio_loop_policy()
 
 
@@ -173,14 +184,16 @@ class ManagerWrapper:
     @cached_property
     def manager(self) -> _manager.Manager:
         try:
-            config = Config.read(self.ctx.params['profile']).ensure_dirs()
+            config = Config.read(None, self.ctx.params['profile']).ensure_dirs()
         except FileNotFoundError:
-            config = self.ctx.invoke(configure, promptless=False)
+            config = self.ctx.invoke(configure)
 
-        setup_logging(config, self.ctx.params['log_level'], self.ctx.params['log_to_stderr'])
+        setup_logging(
+            config.logging_dir, self.ctx.params['log_level'], self.ctx.params['log_to_stderr']
+        )
+        manager, close_db_conn = _manager.Manager.from_config(config)
+        self.ctx.call_on_close(close_db_conn)
 
-        manager = _manager.Manager.from_config(config)
-        self.ctx.call_on_close(manager.database.close)
         return manager
 
     @staticmethod
@@ -192,36 +205,33 @@ class ManagerWrapper:
         return wrapper  # type: ignore
 
 
-def _with_manager(
-    fn: Callable[..., object]
-) -> Callable[[click.Context, click.Parameter, object], object]:
+def _with_manager(fn: Callable[..., object]):
     def wrapper(ctx: click.Context, __: click.Parameter, value: object):
         return fn(ctx.obj.manager, value)
 
     return wrapper
 
 
-class EnumParam(click.Choice):
+class StrEnumParam(click.Choice):
     def __init__(
         self,
-        choice_enum: type[Enum],
-        excludes: Set[Enum] = frozenset(),
+        choice_enum: type[StrEnum],
+        excludes: Set[StrEnum] = frozenset(),
         case_sensitive: bool = True,
     ) -> None:
         self.choice_enum = choice_enum
         super().__init__(
-            choices=[c.name for c in choice_enum if c not in excludes],
+            choices=[c for c in choice_enum if c not in excludes],
             case_sensitive=case_sensitive,
         )
 
     def convert(
         self, value: str, param: click.Parameter | None, ctx: click.Context | None
-    ) -> Enum:
-        parent_result = super().convert(value, param, ctx)
-        return self.choice_enum[parent_result]
+    ) -> StrEnum:
+        return self.choice_enum(super().convert(value, param, ctx))
 
 
-def _register_plugin_commands(group: click.Group) -> click.Group:
+def _register_plugin_commands(group: click.Group):
     plugin_hook = load_plugins()
     additional_commands = (c for g in plugin_hook.instawow_add_commands() for c in g)
     for command in additional_commands:
@@ -229,7 +239,7 @@ def _register_plugin_commands(group: click.Group) -> click.Group:
     return group
 
 
-def _set_log_level(_: click.Context, __: click.Parameter, value: bool) -> str:
+def _set_log_level(_: click.Context, __: click.Parameter, value: bool):
     return 'DEBUG' if value else 'INFO'
 
 
@@ -258,12 +268,15 @@ def _set_log_level(_: click.Context, __: click.Parameter, value: bool) -> str:
     help='Activate the specified profile.',
 )
 @click.pass_context
-def main(ctx: click.Context, log_level: str, log_to_stderr: bool, profile: str) -> None:
+def cli(ctx: click.Context, log_level: str, log_to_stderr: bool, profile: str) -> None:
     "Add-on manager for World of Warcraft."
     _apply_patches()
     ctx.obj = ManagerWrapper(ctx)
 
 
+main = logger.catch(reraise=True)(cli)
+
+
 @overload
 def parse_into_defn(manager: _manager.Manager, value: str, *, raise_invalid: bool = True) -> Defn:
     ...
@@ -304,7 +317,7 @@ def parse_into_defn_with_strategy(
     manager: _manager.Manager, value: Sequence[tuple[Strategy, str]]
 ) -> Iterator[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
-    return map(lambda d, s: d.with_(strategy=s), defns, (s for s, _ in value))
+    return map(lambda d, s: evolve_model_obj(d, strategy=s), defns, (s for s, _ in value))
 
 
 def parse_into_defn_with_version(
@@ -315,11 +328,11 @@ def parse_into_defn_with_version(
 
 
 def _combine_addons(
-    fn: Callable[[_manager.Manager, object], Iterable[Defn]],
+    fn: Callable[[_manager.Manager, _T], Iterable[Defn]],
     ctx: click.Context,
-    __: click.Parameter,
-    value: object,
-) -> None:
+    _: click.Parameter,
+    value: _T,
+):
     addons: list[Defn] = ctx.params.setdefault('addons', [])
     if value:
         addons.extend(fn(ctx.obj.manager, value))
@@ -328,7 +341,7 @@ def _combine_addons(
 _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
 
 
-@main.command()
+@cli.command()
 @click.argument(
     'addons', nargs=-1, callback=partial(_combine_addons, parse_into_defn), expose_value=False
 )
@@ -336,7 +349,7 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
     '--with-strategy',
     '-s',
     multiple=True,
-    type=(EnumParam(Strategy, _EXCLUDED_STRATEGIES), str),
+    type=(StrEnumParam(Strategy, _EXCLUDED_STRATEGIES), str),
     expose_value=False,
     callback=partial(_combine_addons, parse_into_defn_with_strategy),
     metavar='<STRATEGY ADDON>...',
@@ -365,7 +378,7 @@ def install(manager: _manager.Manager, addons: Sequence[Defn], replace: bool) ->
     Report(results.items()).generate_and_exit()
 
 
-@main.command()
+@cli.command()
 @click.argument('addons', nargs=-1, callback=_with_manager(parse_into_defn))
 @ManagerWrapper.pass_manager
 def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
@@ -399,7 +412,7 @@ def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
     Report(results.items(), filter_results).generate_and_exit()
 
 
-@main.command()
+@cli.command()
 @click.argument('addons', nargs=-1, required=True, callback=_with_manager(parse_into_defn))
 @click.option(
     '--keep-folders',
@@ -414,7 +427,7 @@ def remove(manager: _manager.Manager, addons: Sequence[Defn], keep_folders: bool
     Report(results.items()).generate_and_exit()
 
 
-@main.command()
+@cli.command()
 @click.argument('addon', callback=_with_manager(parse_into_defn))
 @click.option(
     '--version',
@@ -432,7 +445,7 @@ def rollback(manager: _manager.Manager, addon: Defn, version: str | None, undo:
     from .prompts import Choice, ask, select
 
     if version and undo:
-        raise click.UsageError('Cannot use "--version" and "--undo" together')
+        raise click.UsageError('Cannot use "--version" with "--undo"')
 
     pkg = manager.get_pkg(addon)
     if not pkg:
@@ -474,25 +487,93 @@ def rollback(manager: _manager.Manager, addon: Defn, version: str | None, undo:
     ).generate_and_exit()
 
 
-@main.command()
+@cli.command()
+@click.option('--auto', '-a', is_flag=True, default=False, help='Do not ask for user input.')
 @click.option(
-    '--auto', '-a', is_flag=True, default=False, help='Do not ask for user confirmation.'
+    '--installed',
+    'rereconcile',
+    is_flag=True,
+    default=False,
+    help='Re-reconcile installed add-ons.',
 )
 @click.option(
     '--list-unreconciled', is_flag=True, default=False, help='List unreconciled add-ons and exit.'
 )
 @ManagerWrapper.pass_manager
-def reconcile(manager: _manager.Manager, auto: bool, list_unreconciled: bool) -> None:
+def reconcile(
+    manager: _manager.Manager, auto: bool, rereconcile: bool, list_unreconciled: bool
+) -> None:
     "Reconcile pre-installed add-ons."
     from .matchers import (
         AddonFolder,
-        get_unreconciled_folder_set,
+        get_unreconciled_folders,
         match_addon_names_with_folder_names,
         match_folder_name_subsets,
         match_toc_source_ids,
     )
     from .prompts import PkgChoice, ask, confirm, select, skip
 
+    def construct_choice(pkg: models.Pkg, highlight_version: bool, disabled: bool):
+        defn = Defn.from_pkg(pkg)
+        return PkgChoice(
+            [
+                ('', f'{defn.to_urn()}=='),
+                ('class:highlight-sub' if highlight_version else '', pkg.version),
+            ],
+            pkg=pkg,
+            value=defn,
+            disabled=disabled,
+        )
+
+    def gather_selections(
+        groups: Sequence[tuple[Any, Sequence[Defn]]],
+        selector: Callable[[Any, Sequence[models.Pkg]], Defn | None],
+    ):
+        results = run_with_progress(manager.resolve(uniq(d for _, b in groups for d in b)))
+        for addons_or_pkg, defns in groups:
+            shortlist = [r for d in defns for r in (results[d],) if isinstance(r, models.Pkg)]
+            if shortlist:
+                selection = selector(addons_or_pkg, shortlist)
+                yield selection
+
+    if rereconcile:
+        if auto:
+            raise click.UsageError('Cannot use "--auto" with "--installed"')
+
+        import sqlalchemy as sa
+
+        def prompt_reconciled(installed_pkg: models.Pkg, pkgs: Sequence[models.Pkg]):
+            highlight_version = not all_eq(i.version for i in (installed_pkg, *pkgs))
+            choices = [
+                construct_choice(installed_pkg, highlight_version, True),
+                *(construct_choice(p, highlight_version, False) for p in pkgs),
+                skip,
+            ]
+            selection = ask(select(installed_pkg.name, choices))
+            return selection or None
+
+        installed_pkgs = (
+            models.Pkg.from_row_mapping(manager.database, p)
+            for p in manager.database.execute(sa.select(db.pkg)).mappings().all()
+        )
+        groups = list(run_with_progress(manager.find_equivalent_pkg_defns(installed_pkgs)).items())
+        selections = [
+            (p, s) for (p, _), s in zip(groups, gather_selections(groups, prompt_reconciled)) if s
+        ]
+        if selections and ask(confirm('Install selected add-ons?')):
+            Report(
+                run_with_progress(
+                    manager.remove([Defn.from_pkg(p) for p, _ in selections], False),
+                ).items()
+            ).generate()
+            Report(
+                run_with_progress(
+                    manager.install([s for _, s in selections], False),
+                ).items()
+            ).generate_and_exit()
+
+        return
+
     preamble = textwrap.dedent(
         '''\
         Use the arrow keys to navigate, <o> to open an add-on in your browser,
@@ -512,42 +593,35 @@ def reconcile(manager: _manager.Manager, auto: bool, list_unreconciled: bool) ->
         '''
     )
 
-    def prompt_one(addons: list[AddonFolder], pkgs: list[models.Pkg]) -> Defn | tuple[()]:
-        def construct_choice(pkg: models.Pkg):
-            defn = Defn.from_pkg(pkg)
-            title = [
-                ('', f'{defn.to_urn()}=='),
-                ('class:highlight-sub' if highlight_version else '', pkg.version),
-            ]
-            return PkgChoice(title, pkg=pkg, value=defn)
+    def prompt_unreconciled(addons: Sequence[AddonFolder], pkgs: Sequence[models.Pkg]):
+        def combine_names():
+            return textwrap.shorten(', '.join(a.name for a in addons), 60)
 
         # Highlight version if there's multiple of them
-        highlight_version = len({i.version for i in chain(addons, pkgs)}) > 1
-        choices = list(chain(map(construct_choice, pkgs), (skip,)))
-        addon = addons[0]
-        selection = ask(select(f'{addon.name} [{addon.version or "?"}]', choices))
-        return selection
+        highlight_version = not all_eq(i.version for i in chain(addons, pkgs))
+        choices = [
+            *(construct_choice(p, highlight_version, False) for p in pkgs),
+            skip,
+        ]
+        selection = ask(select(f'{combine_names()} [{addons[0].version or "?"}]', choices))
+        return selection or None
 
-    def prompt(groups: Sequence[tuple[list[AddonFolder], list[Defn]]]) -> Iterable[Defn]:
-        uniq_defns = uniq(d for _, b in groups for d in b)
-        results = run_with_progress(manager.resolve(uniq_defns))
-        for addons, defns in groups:
-            shortlist: list[models.Pkg] = list(filter(models.is_pkg, (results[d] for d in defns)))
-            if shortlist:
-                selection = Defn.from_pkg(shortlist[0]) if auto else prompt_one(addons, shortlist)
-                selection and (yield selection)
+    def pick_first(addons: Sequence[AddonFolder], pkgs: Sequence[models.Pkg]):
+        return Defn.from_pkg(pkgs[0])
 
-    def match_all() -> Generator[list[Defn], frozenset[AddonFolder], None]:
+    def match_all(
+        selector: Callable[[Any, Sequence[models.Pkg]], Defn | None]
+    ) -> Generator[list[Defn], frozenset[AddonFolder], None]:
         # Match in order of increasing heuristicitivenessitude
-        for fn in (
+        for fn in [
             match_toc_source_ids,
             match_folder_name_subsets,
             match_addon_names_with_folder_names,
-        ):
+        ]:
             groups = run_with_progress(fn(manager, (yield [])))
-            yield list(prompt(groups))
+            yield list(filter(None, gather_selections(groups, selector)))
 
-    leftovers = get_unreconciled_folder_set(manager)
+    leftovers = get_unreconciled_folders(manager)
     if list_unreconciled:
         table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
         click.echo(tabulate(table_rows))
@@ -559,14 +633,14 @@ def reconcile(manager: _manager.Manager, auto: bool, list_unreconciled: bool) ->
     if not auto:
         click.echo(preamble)
 
-    matcher = match_all()
+    matcher = match_all(pick_first if auto else prompt_unreconciled)
     for _ in matcher:  # Skip over consumer yields
         selections = matcher.send(leftovers)
         if selections and (auto or ask(confirm('Install selected add-ons?'))):
-            results = run_with_progress(manager.install(selections, replace=True))
+            results = run_with_progress(manager.install(selections, True))
             Report(results.items()).generate()
 
-        leftovers = get_unreconciled_folder_set(manager)
+        leftovers = get_unreconciled_folders(manager)
 
     if leftovers:
         click.echo()
@@ -574,18 +648,16 @@ def reconcile(manager: _manager.Manager, auto: bool, list_unreconciled: bool) ->
         click.echo(tabulate(table_rows))
 
 
-def _concat_search_terms(_: click.Context, __: click.Parameter, value: tuple[str, ...]) -> str:
+def _concat_search_terms(_: click.Context, __: click.Parameter, value: tuple[str, ...]):
     return ' '.join(value)
 
 
-def _parse_iso_date_into_datetime(
-    _: click.Context, __: click.Parameter, value: str | None
-) -> datetime | None:
+def _parse_iso_date_into_datetime(_: click.Context, __: click.Parameter, value: str | None):
     if value is not None:
         return datetime.strptime(value, '%Y-%m-%d').replace(tzinfo=timezone.utc)
 
 
-@main.command()
+@cli.command()
 @click.argument('search-terms', nargs=-1, required=True, callback=_concat_search_terms)
 @click.option(
     '--limit',
@@ -624,9 +696,9 @@ def search(
     )
     defns = [Defn(e.source, e.id) for e in entries]
     pkgs = (
-        (d.with_(alias=r.slug), r)
+        (evolve_model_obj(d, alias=r.slug), r)
         for d, r in run_with_progress(manager.resolve(defns)).items()
-        if models.is_pkg(r)
+        if isinstance(r, models.Pkg)
     )
     choices = [PkgChoice(f'{p.name}  ({d.to_urn()}=={p.version})', d, pkg=p) for d, p in pkgs]
     if choices:
@@ -637,13 +709,13 @@ def search(
         click.echo('No results found.')
 
 
-class ListFormats(StrEnum):
+class _ListFormats(StrEnum):
     simple = 'simple'
     detailed = 'detailed'
     json = 'json'
 
 
-@main.command('list')
+@cli.command('list')
 @click.argument(
     'addons', nargs=-1, callback=_with_manager(partial(parse_into_defn, raise_invalid=False))
 )
@@ -651,14 +723,14 @@ class ListFormats(StrEnum):
     '--format',
     '-f',
     'output_format',
-    type=EnumParam(ListFormats),
-    default=ListFormats.simple,
+    type=StrEnumParam(_ListFormats),
+    default=_ListFormats.simple,
     show_default=True,
     help='Change the output format.',
 )
 @ManagerWrapper.pass_manager
 def list_installed(
-    manager: _manager.Manager, addons: Sequence[Defn], output_format: ListFormats
+    manager: _manager.Manager, addons: Sequence[Defn], output_format: _ListFormats
 ) -> None:
     "List installed add-ons."
     import sqlalchemy as sa
@@ -699,10 +771,10 @@ def list_installed(
         .all()
     )
 
-    if output_format is ListFormats.json:
+    if output_format is _ListFormats.json:
         click.echo(models.PkgList.parse_obj(list(row_mappings_to_pkgs())).json(indent=2))
 
-    elif output_format is ListFormats.detailed:
+    elif output_format is _ListFormats.detailed:
         formatter = click.HelpFormatter(max_width=99)
         for pkg in row_mappings_to_pkgs():
             with formatter.section(Defn.from_pkg(pkg).to_urn()):
@@ -727,15 +799,15 @@ def list_installed(
         )
 
 
-@main.command(hidden=True)
+@cli.command(hidden=True)
 @click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
 @click.pass_context
 def info(ctx: click.Context, addon: Defn) -> None:
     "Alias of `list -f detailed`."
-    ctx.invoke(list_installed, addons=(addon,), output_format=ListFormats.detailed)
+    ctx.invoke(list_installed, addons=(addon,), output_format=_ListFormats.detailed)
 
 
-@main.command()
+@cli.command()
 @click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
 @ManagerWrapper.pass_manager
 def reveal(manager: _manager.Manager, addon: Defn) -> None:
@@ -747,7 +819,7 @@ def reveal(manager: _manager.Manager, addon: Defn) -> None:
         Report([(addon, R.PkgNotInstalled())]).generate_and_exit()
 
 
-@main.command()
+@cli.command()
 @click.argument(
     'addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)), required=False
 )
@@ -831,16 +903,36 @@ def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool)
         )
 
 
-def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool) -> None:
+def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool):
     if value:
         click.echo(ctx.obj.manager.config.json(indent=2))
         ctx.exit()
 
 
-@main.command()
+async def _github_oauth_flow():
+    from .github_auth import get_codes, poll_for_access_token
+
+    async with _manager.init_web_client() as web_client:
+        codes = await get_codes(web_client)
+        click.echo(f'Navigate to {codes["verification_uri"]} and paste the code below:')
+        click.echo(f'  {codes["user_code"]}')
+        click.echo('Waiting...')
+        access_token = await poll_for_access_token(
+            web_client, codes['device_code'], codes['interval']
+        )
+        return access_token
+
+
+class _EditableConfigOptions(StrEnum):
+    addon_dir = 'addon_dir'
+    game_flavour = 'game_flavour'
+    auto_update_check = 'auto_update_check'
+    github_access_token = 'access_tokens.github'
+
+
+@cli.command()
 @click.option(
-    '--active',
-    'show_active',
+    '--show-active',
     is_flag=True,
     default=False,
     expose_value=False,
@@ -848,20 +940,41 @@ def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool) ->
     callback=_show_active_config,
     help='Show the active configuration and exit.',
 )
-@click.option(
-    '--promptless',
-    is_flag=True,
-    default=False,
-    help='Do not prompt for input and derive the configuration from the environment.',
+@click.argument(
+    'config-options',
+    nargs=-1,
+    type=StrEnumParam(_EditableConfigOptions),
 )
 @click.pass_context
-def configure(ctx: click.Context, promptless: bool) -> Config:
+def configure(
+    ctx: click.Context,
+    config_options: Collection[_EditableConfigOptions],
+) -> Config:
     "Configure instawow."
-    if promptless:
-        make_config = Config
-    else:
-        from .prompts import PydanticValidator, ask, path, select
+    from .prompts import PydanticValidator, ask, confirm, path, select
 
+    if not config_options:
+        config_options = {
+            _EditableConfigOptions.addon_dir,
+            _EditableConfigOptions.game_flavour,
+            _EditableConfigOptions.github_access_token,
+        }
+
+    profile = ctx.find_root().params['profile']
+
+    orig_global_config = GlobalConfig.read()
+    global_config_values = orig_global_config.dict()
+    try:
+        profile_config_values = Config.read(orig_global_config, profile).dict(
+            exclude={'global_config'}
+        )
+    except FileNotFoundError:
+        profile_config_values = {
+            'profile': profile,
+        }
+
+    addon_dir = None
+    if _EditableConfigOptions.addon_dir in config_options:
         addon_dir = ask(
             path(
                 'Add-on directory:',
@@ -869,21 +982,40 @@ def configure(ctx: click.Context, promptless: bool) -> Config:
                 validate=PydanticValidator(Config, 'addon_dir'),
             )
         )
+        profile_config_values['addon_dir'] = addon_dir
+
+    if _EditableConfigOptions.game_flavour in config_options:
         game_flavour = ask(
             select(
                 'Game flavour:',
                 choices=list(Flavour),
-                initial_choice=Config.infer_flavour(addon_dir),
+                initial_choice=Config.infer_flavour(addon_dir) if addon_dir is not None else None,
             )
         )
-        make_config = partial(Config, addon_dir=addon_dir, game_flavour=game_flavour)
+        profile_config_values['game_flavour'] = game_flavour
+
+    if _EditableConfigOptions.auto_update_check in config_options:
+        global_config_values['auto_update_check'] = ask(
+            confirm('Periodically check for instawow updates?')
+        )
+
+    if _EditableConfigOptions.github_access_token in config_options and ask(
+        confirm('Set up GitHub authentication?')
+    ):
+        github_access_token = run_with_progress(_github_oauth_flow())
+        global_config_values['access_tokens']['github'] = github_access_token
+
+    global_config = GlobalConfig(**global_config_values).write()
+    config = Config(global_config=global_config, **profile_config_values).write()
+
+    click.echo('Configuration written to:')
+    click.echo(f'  {global_config.config_file}')
+    click.echo(f'  {config.config_file}')
 
-    config = make_config(profile=ctx.find_root().params['profile']).write()
-    click.echo(f'Configuration written to: {config.config_file}')
     return config
 
 
-@main.group('weakauras-companion')
+@cli.group('weakauras-companion')
 def _weakauras_group() -> None:
     "Manage your WeakAuras."
 
@@ -892,22 +1024,20 @@ def _weakauras_group() -> None:
 @ManagerWrapper.pass_manager
 def build_weakauras_companion(manager: _manager.Manager) -> None:
     "Build the WeakAuras Companion add-on."
-    from .wa_updater import BuilderConfig, WaCompanionBuilder
+    from .wa_updater import WaCompanionBuilder
 
-    config = BuilderConfig()
-    run_with_progress(WaCompanionBuilder(manager, config).build())
+    run_with_progress(WaCompanionBuilder(manager).build())
 
 
 @_weakauras_group.command('list')
 @ManagerWrapper.pass_manager
 def list_installed_wago_auras(manager: _manager.Manager) -> None:
     "List WeakAuras installed from Wago."
-    from .wa_updater import BuilderConfig, WaCompanionBuilder
+    from .wa_updater import WaCompanionBuilder
 
-    config = BuilderConfig()
-    aura_groups = WaCompanionBuilder(manager, config).extract_installed_auras()
+    aura_groups = WaCompanionBuilder(manager).extract_installed_auras()
     installed_auras = sorted(
-        (g.Meta.filename, textwrap.fill(a.id, width=30, max_lines=1), a.url)
+        (g.filename, textwrap.fill(a.id, width=30, max_lines=1), a.url)
         for g in aura_groups
         for v in g.__root__.values()
         for a in v
@@ -916,38 +1046,40 @@ def list_installed_wago_auras(manager: _manager.Manager) -> None:
     click.echo(tabulate([('in file', 'name', 'URL'), *installed_auras]))
 
 
-@main.command(hidden=True)
-@click.argument('filename', type=click.Path(dir_okay=False))
+@cli.command(hidden=True)
 @click.option(
     '--start-date',
     callback=_parse_iso_date_into_datetime,
     help='Omit results before this date.',
     metavar='YYYY-MM-DD',
 )
-def generate_catalogue(filename: str, start_date: datetime | None) -> None:
+def generate_catalogue(start_date: datetime | None) -> None:
     "Generate the master catalogue."
-    from .resolvers import Catalogue
+    from .cataloguer import BaseCatalogue
 
-    catalogue = asyncio.run(Catalogue.collate(start_date))
-    file = Path(filename)
-    file.write_text(
+    catalogue = asyncio.run(BaseCatalogue.collate(start_date))
+    catalogue_path = Path(f'base-catalogue-v{catalogue.version}.json').resolve()
+    catalogue_path.write_text(
         catalogue.json(indent=2),
         encoding='utf-8',
     )
-    file.with_suffix(f'.compact{file.suffix}').write_text(
+    catalogue_path.with_suffix(f'.compact{catalogue_path.suffix}').write_text(
         catalogue.json(separators=(',', ':')),
         encoding='utf-8',
     )
 
 
-@main.command(hidden=importlib.util.find_spec('instawow_gui') is None)
+@cli.command()
 @click.pass_context
 def gui(ctx: click.Context) -> None:
     "Fire up the GUI."
     from instawow_gui.app import InstawowApp
 
-    dummy_config = Config.get_dummy_config(profile='__jsonrpc__').ensure_dirs()
+    global_config = GlobalConfig.read().ensure_dirs()
+    dummy_jsonrpc_config = Config.construct(
+        global_config=global_config, profile='__jsonrpc__'
+    ).ensure_dirs()
     params = ctx.find_root().params
-    setup_logging(dummy_config, params['log_level'], params['log_to_stderr'])
+    setup_logging(dummy_jsonrpc_config.logging_dir, params['log_level'], params['log_to_stderr'])
 
     InstawowApp(version=__version__).main_loop()
