@@ -2,45 +2,34 @@ from __future__ import annotations
 
 import asyncio
 from collections import defaultdict
-from collections.abc import (
-    AsyncIterator,
-    Awaitable,
-    Callable,
-    Iterable,
-    Iterator,
-    Mapping,
-    Sequence,
-    Set,
-)
-from contextlib import asynccontextmanager, contextmanager
+from collections.abc import AsyncIterator, Awaitable, Callable, Iterable, Mapping, Sequence, Set
+from contextlib import AbstractAsyncContextManager, asynccontextmanager, contextmanager
 import contextvars as cv
-import datetime
-from enum import IntEnum
+from datetime import datetime
 from functools import lru_cache, partial, wraps
-from itertools import chain, compress, filterfalse, repeat, starmap, takewhile
+from itertools import chain, filterfalse, repeat, starmap, takewhile
 import json
 from pathlib import Path, PurePath
 from shutil import copy
 from tempfile import NamedTemporaryFile
-from typing import TYPE_CHECKING, Any, TypeVar
+import time
+from typing import Any, TypeVar
 import urllib.parse
 
 from loguru import logger
 import sqlalchemy as sa
-import sqlalchemy.exc as sa_exc
 import sqlalchemy.future as sa_future
 from typing_extensions import Concatenate, Literal, ParamSpec, TypeAlias, TypedDict
 from yarl import URL
 
 from . import _deferred_types, db, models
 from . import results as R
+from .cataloguer import BaseCatalogue, Catalogue, CatalogueEntry
 from .common import Strategy
-from .config import Config
+from .config import Config, GlobalConfig
 from .plugins import load_plugins
 from .resolvers import (
     BaseResolver,
-    Catalogue,
-    CatalogueEntry,
     CurseResolver,
     Defn,
     GithubResolver,
@@ -52,6 +41,7 @@ from .resolvers import (
 from .utils import (
     bucketise,
     chain_dict,
+    evolve_model_obj,
     file_uri_to_path,
     find_addon_zip_base_dirs,
     gather,
@@ -63,14 +53,8 @@ from .utils import (
 from .utils import run_in_thread as t
 from .utils import shasum, trash, uniq
 
-if TYPE_CHECKING:  # pragma: no cover
-    _BaseResolverDict: TypeAlias = 'dict[str, Resolver]'
-else:
-    _BaseResolverDict = dict
-
 _P = ParamSpec('_P')
 _T = TypeVar('_T')
-_CManager: TypeAlias = 'Callable[Concatenate[Manager, _P], Awaitable[_T]]'
 
 
 USER_AGENT = 'instawow (https://github.com/layday/instawow)'
@@ -92,14 +76,14 @@ class _PkgDownloadTraceRequestCtx(TypedDict):
 TraceRequestCtx: TypeAlias = '_GenericDownloadTraceRequestCtx | _PkgDownloadTraceRequestCtx | None'
 
 
-_AsyncNamedTemporaryFile = t(NamedTemporaryFile)
 _copy_async = t(copy)
 _move_async = t(move)
 
 
 @asynccontextmanager
-async def _open_temp_writer() -> AsyncIterator[tuple[Path, Callable[[bytes], Awaitable[int]]]]:
-    fh = await _AsyncNamedTemporaryFile(delete=False)
+async def _open_temp_writer():
+    loop = asyncio.get_running_loop()
+    fh = await loop.run_in_executor(None, lambda: NamedTemporaryFile(delete=False))
     path = Path(fh.name)
     try:
         yield (path, t(fh.write))
@@ -112,7 +96,7 @@ async def _open_temp_writer() -> AsyncIterator[tuple[Path, Callable[[bytes], Awa
 
 
 @contextmanager
-def _open_pkg_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path], None]]]:
+def _open_pkg_archive(path: PurePath):
     from zipfile import ZipFile
 
     with ZipFile(path) as archive:
@@ -125,120 +109,78 @@ def _open_pkg_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path
         yield (base_dirs, extract)
 
 
-async def _download_pkg_archive(
-    manager: Manager, pkg: models.Pkg, *, chunk_size: int = 4096
-) -> Path:
-    url = pkg.download_url
-    dest = manager.config.cache_dir / shasum(
-        pkg.source, pkg.id, pkg.version, manager.config.game_flavour
-    )
-    if await t(dest.exists)():
-        logger.debug(f'{url} is cached at {dest}')
-    elif url.startswith('file://'):
-        await _copy_async(file_uri_to_path(url), dest)
-    else:
-        async with manager.web_client.get(
-            url,
-            raise_for_status=True,
-            trace_request_ctx=_PkgDownloadTraceRequestCtx(
-                report_progress='pkg_download', manager=manager, pkg=pkg
-            ),
-        ) as response, _open_temp_writer() as (temp_path, write):
-            async for chunk in response.content.iter_chunked(chunk_size):
-                await write(chunk)
+class _ResponseWrapper:
+    def __init__(self, response_obj: _deferred_types.aiohttp.ClientResponse | None, text: str):
+        self._response_obj = response_obj
+        self._text = text
 
-        await _move_async(temp_path, dest)
+    async def text(self) -> str:
+        return self._text
 
-    return dest
+    async def json(self, **kwargs: Any) -> Any:
+        return json.loads(await self.text())
+
+    def raise_for_status(self) -> None:
+        if self._response_obj is not None:
+            self._response_obj.raise_for_status()
+
+    @property
+    def status(self) -> int:
+        return self._response_obj.status if self._response_obj is not None else 200
 
 
-async def cache_response(
-    manager: Manager,
-    url: str | URL,
-    ttl: Mapping[str, float],
-    *,
-    label: str | None = None,
-    is_json: bool = True,
-    request_extra: Mapping[str, Any] = {},
-) -> Any:
-    async def make_request():
-        kwargs: dict[str, Any] = {
-            'method': 'GET',
-            'url': url,
-            'raise_for_status': True,
-            **request_extra,
-        }
-        if label:
-            kwargs['trace_request_ctx'] = _GenericDownloadTraceRequestCtx(
-                report_progress='generic', label=label
-            )
-        async with manager.web_client.request(**kwargs) as response:
-            return await response.text()
+class _CacheFauxClientSession:
+    def __init__(self, cache_dir: Path):
+        self._cache_dir = cache_dir
 
-    dest = manager.config.cache_dir / shasum(url, request_extra)
-    if await t(is_not_stale)(dest, ttl):
-        logger.debug(f'{url} is cached at {dest} (ttl: {ttl})')
-        text = await t(dest.read_text)(encoding='utf-8')
-    else:
-        text = await make_request()
-        await t(dest.write_text)(text, encoding='utf-8')
+    @property
+    def wrapped(self) -> _deferred_types.aiohttp.ClientSession:
+        return _web_client.get()
 
-    return json.loads(text) if is_json else text
-
-
-class DatabaseState(IntEnum):
-    current = 0
-    old = 1
-    uninitialised = 2
-
-
-def get_database_state(engine: sa_future.Engine) -> DatabaseState:
-    with engine.connect() as connection:
-        try:
-            state = connection.execute(
-                sa.text(
-                    'SELECT ifnull((SELECT 0 FROM alembic_version WHERE version_num == :version_num), 1)',
-                ),
-                {'version_num': DB_REVISION},
-            ).scalar()
-        except sa_exc.OperationalError:
-            state = connection.execute(
-                sa.text(
-                    'SELECT ifnull((SELECT 1 FROM sqlite_master WHERE type = "table" LIMIT 1), 2)',
+    @asynccontextmanager
+    async def request(
+        self,
+        method: str,
+        url: str | URL,
+        ttl: Mapping[str, float] | None = None,
+        label: str | None = None,
+        **kwargs: Any,
+    ) -> AsyncIterator[_ResponseWrapper]:
+        def prepare_request_kwargs():
+            request_kwargs: dict[str, Any] = {'method': method, 'url': url, **kwargs}
+            if label:
+                request_kwargs['trace_request_ctx'] = _GenericDownloadTraceRequestCtx(
+                    report_progress='generic', label=label
                 )
-            ).scalar()
+            return request_kwargs
 
-    return DatabaseState(state)
+        async def make_request():
+            async with self.wrapped.request(**prepare_request_kwargs()) as response:
+                return (response, await response.text())
 
-
-def migrate_database(engine: sa_future.Engine) -> None:
-    should_migrate = get_database_state(engine)
-    if should_migrate != DatabaseState.current:
-        import alembic.command
-        import alembic.config
-
-        alembic_config = alembic.config.Config()
-        alembic_config.set_main_option('script_location', f'{__package__}:migrations')
-        alembic_config.set_main_option('sqlalchemy.url', str(engine.url))
-
-        if should_migrate == DatabaseState.uninitialised:
-            db.metadata.create_all(engine)
-            alembic.command.stamp(alembic_config, DB_REVISION)
+        if ttl is None:
+            response, text = await make_request()
         else:
-            alembic.command.upgrade(alembic_config, DB_REVISION)
+            dest = self._cache_dir / shasum(url, ttl, kwargs)
+            response = None
+            if await t(is_not_stale)(dest, ttl):
+                logger.debug(f'{url} is cached at {dest} (ttl: {ttl})')
+                text = await t(dest.read_text)(encoding='utf-8')
+            else:
+                response, text = await make_request()
+                if response.ok:
+                    await t(dest.write_text)(text, encoding='utf-8')
 
+        yield _ResponseWrapper(response, text)
 
-def prepare_database(config: Config) -> sa_future.Engine:
-    engine = sa.create_engine(
-        f'sqlite:///{config.db_file}',
-        # We wanna be able to operate on the database from executor threads
-        # when performing disk I/O
-        connect_args={'check_same_thread': False},
-        # echo=True,
-        future=True,
-    )
-    migrate_database(engine)
-    return engine
+    def get(
+        self,
+        url: str | URL,
+        ttl: Mapping[str, float] | None = None,
+        label: str | None = None,
+        **kwargs: Any,
+    ) -> AbstractAsyncContextManager[_ResponseWrapper]:
+        return self.request('GET', url, ttl, label, **kwargs)
 
 
 @lru_cache(None)
@@ -270,7 +212,7 @@ def init_web_client(**kwargs: Any) -> _deferred_types.aiohttp.ClientSession:
         'connector': make_connector(),
         'headers': {'User-Agent': USER_AGENT},
         'trust_env': True,  # Respect the 'http_proxy' env var
-        'timeout': ClientTimeout(connect=60, sock_connect=10, sock_read=10),
+        'timeout': ClientTimeout(connect=60, sock_connect=10, sock_read=20),
         **kwargs,
     }
     return ClientSession(**kwargs)
@@ -286,7 +228,7 @@ class _DummyResolver(BaseResolver):
         return dict.fromkeys(defns, R.PkgSourceInvalid())
 
 
-class _ResolverDict(_BaseResolverDict):
+class _ResolverDict(dict):  # type: ignore
     def __missing__(self, key: str) -> Resolver:
         return _DummyResolver
 
@@ -310,10 +252,10 @@ async def capture_manager_exc_async(
 
 
 class _DummyLock:
-    async def __aenter__(self) -> None:
+    async def __aenter__(self):
         pass
 
-    async def __aexit__(self, *args: object) -> None:
+    async def __aexit__(self, *args: object):
         pass
 
 
@@ -321,10 +263,12 @@ def _with_lock(
     lock_name: str,
     manager_bound: bool = True,
 ):
-    def outer(coro_fn: _CManager[_P, _T]) -> _CManager[_P, _T]:
+    def outer(
+        coro_fn: Callable[Concatenate[Manager, _P], Awaitable[_T]]
+    ) -> Callable[Concatenate[Manager, _P], Awaitable[_T]]:
         @wraps(coro_fn)
         async def inner(self: Manager, *args: _P.args, **kwargs: _P.kwargs):
-            async with self.locks[f'{id(self)}-{lock_name}' if manager_bound else lock_name]:
+            async with self.locks[(id(self), lock_name) if manager_bound else lock_name]:
                 return await coro_fn(self, *args, **kwargs)
 
         return inner
@@ -334,20 +278,41 @@ def _with_lock(
 
 _web_client: cv.ContextVar[_deferred_types.aiohttp.ClientSession] = cv.ContextVar('_web_client')
 
-dummy_locks: defaultdict[str, Any] = defaultdict(_DummyLock)
-_locks: cv.ContextVar[defaultdict[str, asyncio.Lock]] = cv.ContextVar(
-    '_locks', default=dummy_locks
-)
+LocksType: TypeAlias = 'defaultdict[object, AbstractAsyncContextManager[None]]'
+
+_dummy_locks: LocksType = defaultdict(_DummyLock)
+_locks: cv.ContextVar[LocksType] = cv.ContextVar('_locks', default=_dummy_locks)
+
+
+def prepare_database(config: Config) -> sa_future.Engine:
+    engine = sa.create_engine(
+        f'sqlite:///{config.db_file}',
+        # We wanna be able to operate on the database from executor threads
+        # when performing disk I/O
+        connect_args={'check_same_thread': False},
+        # echo=True,
+        future=True,
+    )
+    db.migrate_database(engine, DB_REVISION)
+    return engine
 
 
 class Manager:
-    RESOLVERS = (
+    RESOLVERS = [
+        GithubResolver,
         CurseResolver,
         WowiResolver,
         TukuiResolver,
-        GithubResolver,
         InstawowResolver,
+    ]
+
+    _base_catalogue_url = (
+        f'https://raw.githubusercontent.com/layday/instawow-data/data/'
+        f'base-catalogue-v{BaseCatalogue.construct().version}.compact.json'
     )
+    _catalogue_filename = f'catalogue-v{Catalogue.construct().version}.json'
+
+    _normalise_search_terms = staticmethod(normalise_names(''))
 
     def __init__(
         self,
@@ -361,10 +326,14 @@ class Manager:
         resolver_classes = chain(
             (r for g in plugin_hook.instawow_add_resolvers() for r in g), self.RESOLVERS
         )
-        self.resolvers: _ResolverDict = _ResolverDict(
+        self.resolvers: dict[str, Resolver] = _ResolverDict(
             (r.source, r(self)) for r in resolver_classes
         )
 
+        self.web_client: _CacheFauxClientSession = _CacheFauxClientSession(
+            self.config.global_config.cache_dir
+        )
+
         self._catalogue = None
 
     @classmethod
@@ -372,7 +341,7 @@ class Manager:
         cls,
         *,
         web_client: _deferred_types.aiohttp.ClientSession | None = None,
-        locks: defaultdict[str, asyncio.Lock] | None = None,
+        locks: LocksType | None = None,
     ) -> None:
         if web_client is not None:
             _web_client.set(web_client)
@@ -380,16 +349,12 @@ class Manager:
             _locks.set(locks)
 
     @classmethod
-    def from_config(cls, config: Config) -> Manager:
-        return cls(config, prepare_database(config).connect())
+    def from_config(cls, config: Config) -> tuple[Manager, Callable[[], None]]:
+        db_conn = prepare_database(config).connect()
+        return (cls(config, db_conn), db_conn.close)
 
     @property
-    def web_client(self) -> _deferred_types.aiohttp.ClientSession:
-        "The web client session."
-        return _web_client.get()
-
-    @property
-    def locks(self) -> defaultdict[str, asyncio.Lock]:
+    def locks(self) -> LocksType:
         "Lock factory used to synchronise async operations."
         return _locks.get()
 
@@ -412,7 +377,7 @@ class Manager:
     def check_pkg_exists(self, defn: Defn) -> bool:
         return (
             self.database.execute(
-                sa.select(sa.func.count())
+                sa.select(sa.text('1'))
                 .select_from(db.pkg)
                 .filter(
                     db.pkg.c.source == defn.source,
@@ -421,7 +386,7 @@ class Manager:
                     | (db.pkg.c.slug == defn.alias),
                 )
             ).scalar()
-            != 0
+            == 1
         )
 
     def get_pkg(self, defn: Defn, partial_match: bool = False) -> models.Pkg | None:
@@ -466,7 +431,7 @@ class Manager:
             if replace:
                 trash(
                     [self.config.addon_dir / f for f in top_level_folders],
-                    dest=self.config.temp_dir,
+                    dest=self.config.global_config.temp_dir,
                     missing_ok=True,
                 )
             else:
@@ -478,13 +443,11 @@ class Manager:
 
             extract(self.config.addon_dir)
 
-        pkg = models.Pkg.parse_obj(
-            {**pkg.__dict__, 'folders': [{'name': f} for f in sorted(top_level_folders)]}
-        )
+        pkg = evolve_model_obj(pkg, folders=[{'name': f} for f in sorted(top_level_folders)])
         pkg.insert(self.database)
         return R.PkgInstalled(pkg)
 
-    def update_pkg(self, pkg1: models.Pkg, pkg2: models.Pkg, archive: Path) -> R.PkgUpdated:
+    def update_pkg(self, old_pkg: models.Pkg, new_pkg: models.Pkg, archive: Path) -> R.PkgUpdated:
         "Update a package."
         with _open_pkg_archive(archive) as (top_level_folders, extract):
             installed_conflicts = self.database.execute(
@@ -492,40 +455,41 @@ class Manager:
                 .distinct()
                 .join(db.pkg_folder)
                 .filter(
-                    db.pkg_folder.c.pkg_source != pkg2.source,
-                    db.pkg_folder.c.pkg_id != pkg2.id,
+                    db.pkg_folder.c.pkg_source != new_pkg.source,
+                    db.pkg_folder.c.pkg_id != new_pkg.id,
                     db.pkg_folder.c.name.in_(top_level_folders),
                 )
             ).all()
             if installed_conflicts:
                 raise R.PkgConflictsWithInstalled(installed_conflicts)
 
-            unreconciled_conflicts = top_level_folders - {f.name for f in pkg1.folders} & {
+            unreconciled_conflicts = top_level_folders - {f.name for f in old_pkg.folders} & {
                 f.name for f in self.config.addon_dir.iterdir()
             }
             if unreconciled_conflicts:
                 raise R.PkgConflictsWithUnreconciled(unreconciled_conflicts)
 
             trash(
-                [self.config.addon_dir / f.name for f in pkg1.folders],
-                dest=self.config.temp_dir,
+                [self.config.addon_dir / f.name for f in old_pkg.folders],
+                dest=self.config.global_config.temp_dir,
                 missing_ok=True,
             )
+            old_pkg.delete(self.database)
+
             extract(self.config.addon_dir)
 
-        pkg2 = models.Pkg.parse_obj(
-            {**pkg2.__dict__, 'folders': [{'name': f} for f in sorted(top_level_folders)]}
+        new_pkg = evolve_model_obj(
+            new_pkg, folders=[{'name': f} for f in sorted(top_level_folders)]
         )
-        pkg1.delete(self.database)
-        pkg2.insert(self.database)
-        return R.PkgUpdated(pkg1, pkg2)
+        new_pkg.insert(self.database)
+        return R.PkgUpdated(old_pkg, new_pkg)
 
     def remove_pkg(self, pkg: models.Pkg, keep_folders: bool) -> R.PkgRemoved:
         "Remove a package."
         if not keep_folders:
             trash(
                 [self.config.addon_dir / f.name for f in pkg.folders],
-                dest=self.config.temp_dir,
+                dest=self.config.global_config.temp_dir,
                 missing_ok=True,
             )
 
@@ -535,16 +499,67 @@ class Manager:
     @_with_lock('load catalogue', False)
     async def synchronise(self) -> Catalogue:
         "Fetch the catalogue from the interwebs and load it."
-        if self._catalogue is None:
-            label = 'Synchronising catalogue'
-            url = (
-                'https://raw.githubusercontent.com/layday/instawow-data/data/'
-                'master-catalogue-v4.compact.json'
-            )  # v4
-            raw_catalogue = await cache_response(self, url, {'hours': 4}, label=label)
-            self._catalogue = Catalogue.parse_obj(raw_catalogue)
+        catalogue_json = self.config.global_config.temp_dir / self._catalogue_filename
+        if await t(is_not_stale)(catalogue_json, {'hours': 4}):
+            if self._catalogue is None:
+                raw_catalogue = await t(catalogue_json.read_bytes)()
+                start = time.perf_counter()
+                # Skip validation when loading the catalogue from cache.
+                self._catalogue = Catalogue.from_cache(raw_catalogue)
+                logger.debug(f'loaded catalogue from cache in {time.perf_counter() - start:.3f}s')
+        else:
+            async with self.web_client.get(
+                self._base_catalogue_url,
+                raise_for_status=True,
+                trace_request_ctx=_GenericDownloadTraceRequestCtx(
+                    report_progress='generic', label='Synchronising catalogue'
+                ),
+            ) as response:
+                raw_catalogue = await response.json(content_type=None)
+
+            self._catalogue = Catalogue.from_base_catalogue(raw_catalogue, None)
+            await t(catalogue_json.write_text)(self._catalogue.json(), encoding='utf-8')
+
         return self._catalogue
 
+    async def find_equivalent_pkg_defns(
+        self, pkgs: Iterable[models.Pkg]
+    ) -> dict[models.Pkg, list[Defn]]:
+        "Given a list of packages, find ``Defn``s of each package from other sources."
+        from .matchers import AddonFolder
+
+        catalogue = await self.synchronise()
+
+        def get_catalogue_defns(pkg: models.Pkg) -> frozenset[Defn]:
+            entry = catalogue.keyed_entries.get((pkg.source, pkg.id))
+            if entry:
+                return frozenset(Defn(s.source, s.id) for s in entry.same_as)
+            else:
+                return frozenset()
+
+        def extract_addon_toc_defns(pkg: models.Pkg):
+            return frozenset(
+                d
+                for f in pkg.folders
+                for a in (
+                    AddonFolder.from_addon_path(
+                        self.config.game_flavour, self.config.addon_dir / f.name
+                    ),
+                )
+                if a
+                for d in a.defns_from_toc
+                if d.source != pkg.source
+            )
+
+        resolver_sources = list(self.resolvers.keys())
+
+        return {
+            p: sorted(d, key=lambda d: resolver_sources.index(d.source))
+            for p in pkgs
+            for d in (get_catalogue_defns(p) | await t(extract_addon_toc_defns)(p),)
+            if d
+        }
+
     async def _resolve_deps(self, results: Iterable[Any]) -> dict[Defn, Any]:
         """Resolve package dependencies.
 
@@ -553,7 +568,7 @@ class Manager:
         complexity for something that I would never expect to
         encounter in the wild.
         """
-        pkgs = [r for r in results if models.is_pkg(r)]
+        pkgs = [r for r in results if isinstance(r, models.Pkg)]
         dep_defns = uniq(
             filterfalse(
                 {(p.source, p.id) for p in pkgs}.__contains__,
@@ -565,11 +580,14 @@ class Manager:
 
         deps = await self.resolve(list(starmap(Defn, dep_defns)))
         pretty_deps = {
-            d.with_(alias=r.slug) if models.is_pkg(r) else d: r for d, r in deps.items()
+            evolve_model_obj(d, alias=r.slug) if isinstance(r, models.Pkg) else d: r
+            for d, r in deps.items()
         }
         return pretty_deps
 
-    async def resolve(self, defns: Sequence[Defn], with_deps: bool = False) -> dict[Defn, Any]:
+    async def resolve(
+        self, defns: Sequence[Defn], with_deps: bool = False
+    ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
         "Resolve definitions into packages."
         if not defns:
             return {}
@@ -579,7 +597,7 @@ class Manager:
             (self.resolvers[s].resolve(b) for s, b in defns_by_source.items()),
             capture_manager_exc_async,
         )
-        results_by_defn = chain_dict(
+        results_by_defn: dict[Defn, Any] = chain_dict(
             defns,
             None,
             *(
@@ -597,23 +615,20 @@ class Manager:
         if url.scheme == 'data' and url.raw_path.startswith(','):
             return urllib.parse.unquote(url.raw_path[1:])
         elif url.scheme in {'http', 'https'}:
-            return await cache_response(
-                self,
-                url,
-                {'days': 1},
-                is_json=False,
-            )
+            async with self.web_client.get(url, {'days': 1}, raise_for_status=True) as response:
+                return await response.text()
         elif url.scheme == 'file':
             return await t(Path(file_uri_to_path(uri)).read_text)(encoding='utf-8')
         else:
-            raise ValueError('Unsupported URL with scheme', url.scheme)
+            raise ValueError('Unsupported URI with scheme', url.scheme)
 
     async def search(
         self,
         search_terms: str,
         limit: int,
         sources: Set[str] = frozenset(),
-        start_date: datetime.datetime | None = None,
+        start_date: datetime | None = None,
+        installed_only: bool = False,
     ) -> list[CatalogueEntry]:
         "Search the master catalogue for packages by name."
         import rapidfuzz
@@ -623,7 +638,7 @@ class Manager:
         ew = 0.5
         dw = 1 - ew
 
-        normalise = normalise_names('')
+        threshold = 0 if search_terms == '*' else 70
 
         if not sources:
             sources = self.resolvers.keys()
@@ -655,14 +670,28 @@ class Manager:
 
         filter_fns = list(make_filter())
 
-        s = normalise(search_terms)
+        s = self._normalise_search_terms(search_terms)
+
+        if installed_only:
+            entries = (
+                e
+                for p in self.database.execute(sa.select(db.pkg.c.source, db.pkg.c.id)).all()
+                for e in (catalogue.keyed_entries.get(p),)
+                if e
+            )
+        else:
+            entries = catalogue.entries
 
         tokens_to_entries = bucketise(
-            ((normalise(e.name), e) for e in catalogue.__root__ if all(f(e) for f in filter_fns)),
+            ((e.normalised_name, e) for e in entries if all(f(e) for f in filter_fns)),
             key=lambda v: v[0],
         )
         matches = rapidfuzz.process.extract(
-            s, list(tokens_to_entries), scorer=rapidfuzz.fuzz.WRatio, limit=limit * 2
+            s,
+            list(tokens_to_entries),
+            scorer=rapidfuzz.fuzz.WRatio,
+            limit=limit * 2,
+            score_cutoff=threshold,
         )
         weighted_entries = sorted(
             (
@@ -674,6 +703,29 @@ class Manager:
         )
         return [e for _, e in weighted_entries[:limit]]
 
+    async def _download_pkg_archive(self, pkg: models.Pkg, *, chunk_size: int = 4096):
+        url = pkg.download_url
+        dest = self.config.global_config.cache_dir / shasum(url)
+
+        if await t(dest.exists)():
+            logger.debug(f'{url} is cached at {dest}')
+        elif url.startswith('file://'):
+            await _copy_async(file_uri_to_path(url), dest)
+        else:
+            async with self.web_client.wrapped.get(
+                url,
+                raise_for_status=True,
+                trace_request_ctx=_PkgDownloadTraceRequestCtx(
+                    report_progress='pkg_download', manager=self, pkg=pkg
+                ),
+            ) as response, _open_temp_writer() as (temp_path, write):
+                async for chunk in response.content.iter_chunked(chunk_size):
+                    await write(chunk)
+
+            await _move_async(temp_path, dest)
+
+        return dest
+
     @_with_lock('change state')
     async def install(
         self, defns: Sequence[Defn], replace: bool
@@ -683,27 +735,27 @@ class Manager:
         # doing it this way isn't particularly efficient but avoids having to
         # deal with local state in `resolve()`
         resolve_results = await self.resolve(
-            list(compress(defns, (not self.check_pkg_exists(d) for d in defns))),
-            with_deps=True,
+            [d for d in defns if not self.check_pkg_exists(d)], with_deps=True
         )
-        resolve_results = dict(
-            compress(
-                resolve_results.items(), (not self.check_pkg_exists(d) for d in resolve_results)
-            )
-        )
-        installables = {d: r for d, r in resolve_results.items() if models.is_pkg(r)}
+        resolve_results = {
+            d: r for d, r in resolve_results.items() if not self.check_pkg_exists(d)
+        }
+        installables = {d: r for d, r in resolve_results.items() if isinstance(r, models.Pkg)}
         archives = await gather(
-            (_download_pkg_archive(self, r) for r in installables.values()),
+            (self._download_pkg_archive(r) for r in installables.values()),
             capture_manager_exc_async,
         )
-        results = chain_dict(
+        results: dict[Defn, Any] = chain_dict(
             defns,
             R.PkgAlreadyInstalled(),
             resolve_results.items(),
             [
-                (d, await capture_manager_exc_async(t(self.install_pkg)(p, a, replace)))
-                if isinstance(a, PurePath)
-                else (d, a)
+                (
+                    d,
+                    await capture_manager_exc_async(t(self.install_pkg)(p, a, replace))
+                    if isinstance(a, PurePath)
+                    else a,
+                )
                 for (d, p), a in zip(installables.items(), archives)
             ],
         )
@@ -725,14 +777,14 @@ class Manager:
             # corresponding installed package.  Using the ID has the benefit
             # of resolving installed-but-renamed packages - the slug is
             # transient but the ID isn't
-            d.with_(id=p.id) if retain_defn_strategy else Defn.from_pkg(p): d
+            evolve_model_obj(d, id=p.id) if retain_defn_strategy else Defn.from_pkg(p): d
             for d, p in defns_to_pkgs.items()
         }
         # Discard the reconstructed ``Defn``s
         resolve_results = {
             resolve_defns[d]: r for d, r in (await self.resolve(list(resolve_defns))).items()
         }
-        installables = {d: r for d, r in resolve_results.items() if models.is_pkg(r)}
+        installables = {d: r for d, r in resolve_results.items() if isinstance(r, models.Pkg)}
         updatables = {
             d: (o, n)
             for d, n in installables.items()
@@ -740,10 +792,10 @@ class Manager:
             if n.version != o.version
         }
         archives = await gather(
-            (_download_pkg_archive(self, n) for _, n in updatables.values()),
+            (self._download_pkg_archive(n) for _, n in updatables.values()),
             capture_manager_exc_async,
         )
-        results = chain_dict(
+        results: dict[Defn, Any] = chain_dict(
             defns,
             R.PkgNotInstalled(),
             resolve_results.items(),
@@ -752,9 +804,12 @@ class Manager:
                 for d, p in installables.items()
             ),
             [
-                (d, await capture_manager_exc_async(t(self.update_pkg)(o, n, a)))
-                if isinstance(a, PurePath)
-                else (d, a)
+                (
+                    d,
+                    await capture_manager_exc_async(t(self.update_pkg)(o, n, a))
+                    if isinstance(a, PurePath)
+                    else a,
+                )
                 for (d, (o, n)), a in zip(updatables.items(), archives)
             ],
         )
@@ -842,11 +897,11 @@ async def is_outdated() -> tuple[bool, str]:
     if __version__ == '0.0.0':
         return (False, '')
 
-    dummy_config = Config.get_dummy_config()
-    if not dummy_config.auto_update_check:
+    global_config = await t(GlobalConfig.read)()
+    if not global_config.auto_update_check:
         return (False, '')
 
-    cache_file = dummy_config.temp_dir / '.pypi_version'
+    cache_file = global_config.temp_dir / '.pypi_version'
     if await t(is_not_stale)(cache_file, {'days': 1}):
         version = cache_file.read_text(encoding='utf-8')
     else:
