@@ -1,42 +1,33 @@
 from __future__ import annotations
 
+import asyncio
 from collections.abc import AsyncIterator, Sequence, Set
 from datetime import datetime, timezone
 from enum import IntEnum
-from itertools import chain, takewhile
+from functools import partial
+from itertools import takewhile
 import re
 import typing
-from typing import TYPE_CHECKING, Any, ClassVar
+from typing import Any, ClassVar
 
 from loguru import logger
 from pydantic import BaseModel
-from typing_extensions import Literal, Protocol, TypedDict
+from typing_extensions import Literal
+from typing_extensions import NotRequired as N
+from typing_extensions import Protocol, TypedDict
 from yarl import URL
 
 from . import _deferred_types, manager, models
 from . import results as R
-from .common import Strategy
-from .config import Flavour
-from .utils import StrEnum, bucketise, cached_property, gather, normalise_names, uniq
-
-if TYPE_CHECKING:  # pragma: no cover
-    from typing_extensions import NotRequired as N
+from .cataloguer import BaseCatatalogueEntry
+from .common import ChangelogFormat, Flavour, Strategy
+from .utils import evolve_model_obj, gather, normalise_names, uniq
 
 
-class ChangelogFormat(StrEnum):
-    html = 'html'
-    markdown = 'markdown'
-    bbcode = 'bbcode'
-    raw = 'raw'
-
-
-class Defn(
-    BaseModel,
-    frozen=True,
-):
+class Defn(BaseModel, frozen=True):
     source: str
-    alias: str
     id: typing.Optional[str] = None
+    alias: str
     strategy: Strategy = Strategy.default
     version: typing.Optional[str] = None
 
@@ -46,18 +37,11 @@ class Defn(
     @classmethod
     def from_pkg(cls, pkg: models.Pkg) -> Defn:
         return cls(
-            source=pkg.source,
-            alias=pkg.slug,
-            id=pkg.id,
-            strategy=pkg.options.strategy,
-            version=pkg.version,
+            pkg.source, pkg.slug, id=pkg.id, strategy=pkg.options.strategy, version=pkg.version
         )
 
-    def with_(self, **kwargs: Any) -> Defn:
-        return self.__class__(**{**self.__dict__, **kwargs})
-
     def with_version(self, version: str) -> Defn:
-        return self.with_(strategy=Strategy.version, version=version)
+        return evolve_model_obj(self, strategy=Strategy.version, version=version)
 
     def to_urn(self) -> str:
         return f'{self.source}:{self.alias}'
@@ -72,54 +56,6 @@ def _format_data_changelog(changelog: str = '') -> str:
     return f'data:,{urllib.parse.quote(changelog)}'
 
 
-class CatatalogueBaseEntry(BaseModel):
-    source: str
-    id: str
-    slug: str = ''
-    name: str
-    game_flavours: typing.Set[Flavour]
-    folders: typing.List[typing.Set[str]] = []
-    download_count: int
-    last_updated: datetime
-
-
-class CatalogueEntry(CatatalogueBaseEntry):
-    derived_download_score: float
-
-
-class Catalogue(
-    BaseModel,
-    json_encoders={set: sorted},
-    keep_untouched=(cached_property,),
-):
-    __root__: typing.List[CatalogueEntry]
-
-    @classmethod
-    async def collate(cls, start_date: datetime | None) -> Catalogue:
-        async with manager.init_web_client() as web_client:
-            items = [a for r in manager.Manager.RESOLVERS async for a in r.catalogue(web_client)]
-
-        most_downloads_per_source = {
-            s: max(e.download_count for e in i)
-            for s, i in bucketise(items, key=lambda v: v.source).items()
-        }
-        entries = (
-            CatalogueEntry(
-                **i.__dict__,
-                derived_download_score=i.download_count / most_downloads_per_source[i.source],
-            )
-            for i in items
-        )
-        if start_date is not None:
-            entries = (e for e in entries if e.last_updated >= start_date)
-        catalogue = cls.parse_obj(list(entries))
-        return catalogue
-
-    @cached_property
-    def curse_slugs(self) -> dict[str, str]:
-        return {a.slug: a.id for a in self.__root__ if a.source == 'curse'}
-
-
 class Resolver(Protocol):
     source: ClassVar[str]
     name: ClassVar[str]
@@ -132,19 +68,22 @@ class Resolver(Protocol):
     @staticmethod
     def get_alias_from_url(url: URL) -> str | None:
         "Attempt to extract a ``Defn`` alias from a given URL."
+        ...
 
     async def resolve(
         self, defns: Sequence[Defn]
     ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
         "Resolve multiple ``Defn``s into packages."
+        ...
 
     async def resolve_one(self, defn: Defn, metadata: Any) -> models.Pkg:
         "Resolve a ``Defn`` into a package."
+        ...
 
     @classmethod
     async def catalogue(
         cls, web_client: _deferred_types.aiohttp.ClientSession
-    ) -> AsyncIterator[CatatalogueBaseEntry]:
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
         "Yield add-ons from source for cataloguing."
         return
         yield
@@ -160,10 +99,9 @@ class BaseResolver(Resolver):
         async def resolve_one(self: BaseResolver, defn: Defn, metadata: Any) -> models.Pkg:
             if defn.strategy not in self.strategies:
                 raise R.PkgStrategyUnsupported(defn.strategy)
-
             return await orig_resolve_one(self, defn, metadata)
 
-        cls.resolve_one = resolve_one  # type: ignore
+        setattr(cls, 'resolve_one', resolve_one)
 
     @staticmethod
     def get_alias_from_url(url: URL) -> str | None:
@@ -183,7 +121,7 @@ class BaseResolver(Resolver):
     @classmethod
     async def catalogue(
         cls, web_client: _deferred_types.aiohttp.ClientSession
-    ) -> AsyncIterator[CatatalogueBaseEntry]:
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
         return
         yield
 
@@ -285,7 +223,6 @@ class CurseResolver(BaseResolver):
     )
     changelog_format = ChangelogFormat.html
 
-    # Reference: https://twitchappapi.docs.apiary.io/
     addon_api_url = URL('https://addons-ecs.forgesvc.net/api/v2/addon')
 
     @staticmethod
@@ -300,27 +237,31 @@ class CurseResolver(BaseResolver):
     async def resolve(
         self, defns: Sequence[Defn]
     ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
-        from aiohttp import ClientResponseError
-
         catalogue = await self.manager.synchronise()
 
-        defns_to_ids = {d: d.id or catalogue.curse_slugs.get(d.alias) or d.alias for d in defns}
-        numeric_ids = uniq(i for i in defns_to_ids.values() if i.isdigit())
-        try:
-            json_response: list[_CurseAddon] = await manager.cache_response(
-                self.manager,
-                self.addon_api_url,
-                {'minutes': 5},
-                request_extra={'method': 'POST', 'json': numeric_ids},
-            )
-        except ClientResponseError as error:
-            if error.status != 404:
-                raise
-            json_response = []
+        defns_to_maybe_numeric_ids = {
+            d: i if i.isdigit() else None
+            for d in defns
+            for i in (d.id or catalogue.curse_slugs.get(d.alias) or d.alias,)
+        }
+        async with self.manager.web_client.request(
+            'POST',
+            self.addon_api_url,
+            {'minutes': 5},
+            json=[i for i in defns_to_maybe_numeric_ids.values() if i is not None],
+        ) as response:
+            if response.status == 404:
+                addons: list[_CurseAddon] = []
+            else:
+                response.raise_for_status()
+                addons = await response.json()
 
-        api_results = {str(r['id']): r for r in json_response}
+        numeric_ids_to_addons = {str(r['id']): r for r in addons}
         results = await gather(
-            (self.resolve_one(d, api_results.get(i)) for d, i in defns_to_ids.items()),
+            (
+                self.resolve_one(d, numeric_ids_to_addons.get(i) if i is not None else i)
+                for d, i in defns_to_maybe_numeric_ids.items()
+            ),
             manager.capture_manager_exc_async,
         )
         return dict(zip(defns, results))
@@ -330,12 +271,13 @@ class CurseResolver(BaseResolver):
             raise R.PkgNonexistent
 
         if defn.strategy is Strategy.version:
-            all_files: list[_CurseFile] = await manager.cache_response(
-                self.manager,
+            async with self.manager.web_client.get(
                 self.addon_api_url / str(metadata['id']) / 'files',
                 {'hours': 1},
                 label=f'Fetching metadata from {self.name}',
-            )
+            ) as response:
+                response.raise_for_status()
+                all_files: list[_CurseFile] = await response.json()
 
             file = next((f for f in all_files if defn.version == f['displayName']), None)
             if file is None:
@@ -405,27 +347,33 @@ class CurseResolver(BaseResolver):
                     f'using {defn.strategy} strategy'
                 )
 
-        return models.Pkg(
-            source=self.source,
-            id=metadata['id'],
-            slug=metadata['slug'],
-            name=metadata['name'],
-            description=metadata['summary'],
-            url=metadata['websiteUrl'],
-            download_url=file['downloadUrl'],
-            date_published=file['fileDate'],
-            version=file['displayName'],
-            changelog_url=str(
-                self.addon_api_url / str(metadata['id']) / 'file' / str(file['id']) / 'changelog'
-            ),
-            options={'strategy': defn.strategy},
-            deps=[{'id': d['addonId']} for d in file['dependencies'] if d['type'] == 3],
+        return models.Pkg.parse_obj(
+            {
+                'source': self.source,
+                'id': metadata['id'],
+                'slug': metadata['slug'],
+                'name': metadata['name'],
+                'description': metadata['summary'],
+                'url': metadata['websiteUrl'],
+                'download_url': file['downloadUrl'],
+                'date_published': file['fileDate'],
+                'version': file['displayName'],
+                'changelog_url': str(
+                    self.addon_api_url
+                    / str(metadata['id'])
+                    / 'file'
+                    / str(file['id'])
+                    / 'changelog'
+                ),
+                'options': {'strategy': defn.strategy},
+                'deps': [{'id': d['addonId']} for d in file['dependencies'] if d['type'] == 3],
+            }
         )
 
     @classmethod
     async def catalogue(
         cls, web_client: _deferred_types.aiohttp.ClientSession
-    ) -> AsyncIterator[CatatalogueBaseEntry]:
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
         def supports_x(files: list[_CurseAddon_File]):
             def excise_flavour(
                 curse_flavor: _CurseFlavor,
@@ -433,10 +381,13 @@ class CurseResolver(BaseResolver):
                 flavour: Flavour,
             ):
                 if any(
-                    f['gameVersionFlavor'] == curse_flavor
-                    or any(
-                        s.get('gameVersionTypeId') == curse_type_id
-                        for s in f['sortableGameVersion']
+                    not f['exposeAsAlternative']
+                    and (
+                        f['gameVersionFlavor'] == curse_flavor
+                        or any(
+                            s.get('gameVersionTypeId') == curse_type_id
+                            for s in f['sortableGameVersion']
+                        )
                     )
                     for f in files
                 ):
@@ -458,34 +409,38 @@ class CurseResolver(BaseResolver):
                 Flavour.burning_crusade_classic,
             )
 
+        def get_folders(files: list[_CurseAddon_File]):
+            return uniq(frozenset(m['foldername'] for m in f['modules']) for f in files)
+
         step = 50
         sort_order = '3'  # Alphabetical
         for index in range(0, 10001 - step, step):
-            async with web_client.get(
-                (cls.addon_api_url / 'search').with_query(
-                    gameId='1', sort=sort_order, pageSize=step, index=index
-                )
-            ) as response:
+            # Try not to get rate limited
+            await asyncio.sleep(2)
+
+            url = (cls.addon_api_url / 'search').with_query(
+                gameId='1', sort=sort_order, pageSize=step, index=index
+            )
+            logger.debug(f'retrieving {url}')
+            async with web_client.get(url) as response:
                 items: list[_CurseAddon] = await response.json()
 
             if not items:
                 break
 
             for item in items:
-                folders = uniq(
-                    frozenset(m['foldername'] for m in f['modules'])
-                    for f in item['latestFiles']
-                    if not f['exposeAsAlternative']
-                )
-                yield CatatalogueBaseEntry(
-                    source=cls.source,
-                    id=item['id'],
-                    slug=item['slug'],
-                    name=item['name'],
-                    game_flavours=supports_x(item['latestFiles']),
-                    folders=folders,
-                    download_count=item['downloadCount'],
-                    last_updated=item['dateReleased'],
+                yield BaseCatatalogueEntry.parse_obj(
+                    {
+                        'source': cls.source,
+                        'id': item['id'],
+                        'slug': item['slug'],
+                        'name': item['name'],
+                        'url': item['websiteUrl'],
+                        'game_flavours': supports_x(item['latestFiles']),
+                        'download_count': item['downloadCount'],
+                        'last_updated': item['dateReleased'],
+                        'folders': get_folders(item['latestFiles']),
+                    }
                 )
 
 
@@ -570,35 +525,33 @@ class WowiResolver(BaseResolver):
                 match = re.match(r'^(?:download|info)(?P<id>\d+)', url.name)
                 return match and match['id']
 
-    async def _synchronise(self) -> dict[str, _WowiListApiItem]:
+    async def _synchronise(self):
         async with self.manager.locks['load WoWI catalogue']:
-            list_api_items = await manager.cache_response(
-                self.manager,
+            async with self.manager.web_client.get(
                 self.list_api_url,
                 {'hours': 1},
                 label=f'Synchronising {self.name} catalogue',
-            )
-            return {i['UID']: i for i in list_api_items}
+            ) as response:
+                response.raise_for_status()
+                list_api_items: list[_WowiListApiItem] = await response.json()
+                return {i['UID']: i for i in list_api_items}
 
     async def resolve(
         self, defns: Sequence[Defn]
     ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
-        from aiohttp import ClientResponseError
-
         list_api_items = await self._synchronise()
 
         defns_to_ids = {d: ''.join(takewhile(str.isdigit, d.alias)) for d in defns}
         numeric_ids = frozenset(filter(None, defns_to_ids.values()))
-        try:
-            details_api_items: list[_WowiDetailsApiItem] = await manager.cache_response(
-                self.manager,
-                self.details_api_url / f'{",".join(numeric_ids)}.json',
-                {'minutes': 5},
-            )
-        except ClientResponseError as error:
-            if error.status != 404:
-                raise
-            details_api_items = []
+        async with self.manager.web_client.get(
+            self.details_api_url / f'{",".join(numeric_ids)}.json',
+            {'minutes': 5},
+        ) as response:
+            if response.status == 404:
+                details_api_items: list[_WowiDetailsApiItem] = []
+            else:
+                response.raise_for_status()
+                details_api_items = await response.json()
 
         combined_items: dict[str, Any] = {
             i['UID']: {**list_api_items[i['UID']], **i} for i in details_api_items
@@ -613,38 +566,44 @@ class WowiResolver(BaseResolver):
         if metadata is None:
             raise R.PkgNonexistent
 
-        return models.Pkg(
-            source=self.source,
-            id=metadata['UID'],
-            slug=slugify(f'{metadata["UID"]} {metadata["UIName"]}'),
-            name=metadata['UIName'],
-            description=metadata['UIDescription'],
-            url=metadata['UIFileInfoURL'],
-            download_url=metadata['UIDownload'],
-            date_published=metadata['UIDate'],
-            version=metadata['UIVersion'],
-            changelog_url=_format_data_changelog(metadata['UIChangeLog']),
-            options={'strategy': defn.strategy},
+        return models.Pkg.parse_obj(
+            {
+                'source': self.source,
+                'id': metadata['UID'],
+                'slug': slugify(f'{metadata["UID"]} {metadata["UIName"]}'),
+                'name': metadata['UIName'],
+                'description': metadata['UIDescription'],
+                'url': metadata['UIFileInfoURL'],
+                'download_url': metadata['UIDownload'],
+                'date_published': metadata['UIDate'],
+                'version': metadata['UIVersion'],
+                'changelog_url': _format_data_changelog(metadata['UIChangeLog']),
+                'options': {'strategy': defn.strategy},
+            }
         )
 
     @classmethod
     async def catalogue(
         cls, web_client: _deferred_types.aiohttp.ClientSession
-    ) -> AsyncIterator[CatatalogueBaseEntry]:
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
         flavours = set(Flavour)
 
+        logger.debug(f'retrieving {cls.list_api_url}')
         async with web_client.get(cls.list_api_url) as response:
             items: list[_WowiListApiItem] = await response.json()
 
         for item in items:
-            yield CatatalogueBaseEntry(
-                source=cls.source,
-                id=item['UID'],
-                name=item['UIName'],
-                folders=[item['UIDir']],
-                game_flavours=flavours,
-                download_count=item['UIDownloadTotal'],
-                last_updated=item['UIDate'],
+            yield BaseCatatalogueEntry.parse_obj(
+                {
+                    'source': cls.source,
+                    'id': item['UID'],
+                    'name': item['UIName'],
+                    'url': item['UIFileInfoURL'],
+                    'game_flavours': flavours,
+                    'download_count': item['UIDownloadTotal'],
+                    'last_updated': datetime.fromtimestamp(item['UIDate'] / 1000, timezone.utc),
+                    'folders': [item['UIDir']],
+                }
             )
 
 
@@ -716,20 +675,22 @@ class TukuiResolver(BaseResolver):
 
     async def _synchronise(self) -> dict[str, _TukuiAddon | _TukuiUi]:
         async def fetch_ui(ui_slug: str):
-            addon: _TukuiUi = await manager.cache_response(
-                self.manager,
+            async with self.manager.web_client.get(
                 self.api_url.with_query({'ui': ui_slug}),
                 {'minutes': 5},
-            )
+            ) as response:
+                response.raise_for_status()
+                addon: _TukuiUi = await response.json()
             return [(str(addon['id']), addon), (ui_slug, addon)]
 
         async def fetch_addons(flavour: Flavour):
-            addons: list[_TukuiAddon] = await manager.cache_response(
-                self.manager,
+            async with self.manager.web_client.get(
                 self.api_url.with_query({self.query_flavours[flavour]: 'all'}),
                 {'minutes': 30},
                 label=f'Synchronising {self.name} {flavour} catalogue',
-            )
+            ) as response:
+                response.raise_for_status()
+                addons: list[_TukuiAddon] = await response.json()
             return [(str(a['id']), a) for a in addons]
 
         async with self.manager.locks['load Tukui catalogue']:
@@ -772,63 +733,66 @@ class TukuiResolver(BaseResolver):
         else:
             slug = slugify(f'{metadata["id"]} {metadata["name"]}')
 
-        return models.Pkg(
-            source=self.source,
-            id=str(metadata['id']),
-            slug=slug,
-            name=metadata['name'],
-            description=metadata['small_desc'],
-            url=metadata['web_url'],
-            download_url=metadata['url'],
-            date_published=datetime.fromisoformat(metadata['lastupdate']).replace(
-                tzinfo=timezone.utc
-            ),
-            version=metadata['version'],
-            changelog_url=(
-                # The changelog URL is not versioned - adding fragment to allow caching
-                str(URL(metadata['changelog']).with_fragment(metadata['version']))
-                if metadata['id'] in {-1, -2}
-                # Regular add-ons don't have dedicated changelogs but rather
-                # link to the changelog tab on the add-on page
-                else _format_data_changelog(metadata['changelog'])
-            ),
-            options={'strategy': defn.strategy},
+        return models.Pkg.parse_obj(
+            {
+                'source': self.source,
+                'id': str(metadata['id']),
+                'slug': slug,
+                'name': metadata['name'],
+                'description': metadata['small_desc'],
+                'url': metadata['web_url'],
+                'download_url': metadata['url'],
+                'date_published': datetime.fromisoformat(metadata['lastupdate']).replace(
+                    tzinfo=timezone.utc
+                ),
+                'version': metadata['version'],
+                'changelog_url': (
+                    # The changelog URL is not versioned - adding fragment to allow caching
+                    str(URL(metadata['changelog']).with_fragment(metadata['version']))
+                    if metadata['id'] in {-1, -2}
+                    # Regular add-ons don't have dedicated changelogs but rather
+                    # link to the changelog tab on the add-on page
+                    else _format_data_changelog(metadata['changelog'])
+                ),
+                'options': {'strategy': defn.strategy},
+            }
         )
 
     @classmethod
     async def catalogue(
         cls, web_client: _deferred_types.aiohttp.ClientSession
-    ) -> AsyncIterator[CatatalogueBaseEntry]:
-        async def fetch_ui(ui_slug: str) -> list[_TukuiUi]:
-            async with web_client.get(cls.api_url.with_query({'ui': ui_slug})) as response:
-                return [await response.json(content_type=None)]  # text/html
-
-        async def fetch_addons(query: str) -> list[_TukuiAddon]:
-            async with web_client.get(cls.api_url.with_query({query: 'all'})) as response:
-                return await response.json(content_type=None)  # text/html
-
-        for flavours, item_coro in [
-            ({Flavour.retail}, fetch_ui('tukui')),
-            ({Flavour.retail}, fetch_ui('elvui')),
-            ({Flavour.retail}, fetch_addons('addons')),
-            ({Flavour.vanilla_classic}, fetch_addons('classic-addons')),
-            ({Flavour.burning_crusade_classic}, fetch_addons('classic-tbc-addons')),
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
+        for flavours, query in [
+            ({Flavour.retail}, {'ui': 'tukui'}),
+            ({Flavour.retail}, {'ui': 'elvui'}),
+            *(({f}, {q: 'all'}) for f, q in cls.query_flavours.items()),
         ]:
-            for item in await item_coro:
-                yield CatatalogueBaseEntry(
-                    source=cls.source,
-                    id=item['id'],
-                    name=item['name'],
-                    game_flavours=flavours,
-                    # Split Tukui and ElvUI downloads evenly between them.
-                    # They both have the exact same number of downloads so
-                    # I'm assuming they're being counted together.
-                    # This should help with scoring other add-ons on the
-                    # Tukui catalogue higher
-                    download_count=int(item['downloads']) // (2 if item['id'] in {-1, -2} else 1),
-                    last_updated=datetime.fromisoformat(item['lastupdate']).replace(
-                        tzinfo=timezone.utc
-                    ),
+            url = cls.api_url.with_query(query)
+            logger.debug(f'retrieving {url}')
+            async with web_client.get(url) as response:
+                items: _TukuiUi | list[_TukuiAddon] = await response.json(
+                    content_type=None  # text/html
+                )
+
+            for item in items if isinstance(items, list) else [items]:
+                yield BaseCatatalogueEntry.parse_obj(
+                    {
+                        'source': cls.source,
+                        'id': item['id'],
+                        'name': item['name'],
+                        'url': item['web_url'],
+                        'game_flavours': flavours,
+                        # Split Tukui and ElvUI downloads evenly between them.
+                        # They both have the exact same number of downloads so
+                        # I'm assuming they're being counted together.
+                        # This should help with scoring other add-ons on the
+                        # Tukui catalogue higher
+                        'download_count': int(item['downloads'])
+                        // (2 if item['id'] in {-1, -2} else 1),
+                        'last_updated': datetime.fromisoformat(item['lastupdate']).replace(
+                            tzinfo=timezone.utc
+                        ),
+                    }
                 )
 
 
@@ -878,11 +842,29 @@ class GithubResolver(BaseResolver):
 
     repos_api_url = URL('https://api.github.com/repos')
 
-    release_json_flavours = {
-        Flavour.retail: 'mainline',
-        Flavour.vanilla_classic: 'classic',
-        Flavour.burning_crusade_classic: 'bcc',
-    }
+    generated_catalogue_csv_url = (
+        'https://raw.githubusercontent.com/layday/github-wow-addon-catalogue/main/addons.csv'
+    )
+
+    @staticmethod
+    def _flavour_to_release_json_flavour(flavour: Flavour):
+        if flavour is Flavour.retail:
+            return 'mainline'
+        elif flavour is Flavour.vanilla_classic:
+            return 'classic'
+        elif flavour is Flavour.burning_crusade_classic:
+            return 'bcc'
+
+    @staticmethod
+    def _release_json_flavour_to_flavour(flavour: str):
+        if flavour == 'mainline':
+            return Flavour.retail
+        elif flavour == 'classic':
+            return Flavour.vanilla_classic
+        elif flavour == 'bcc':
+            return Flavour.burning_crusade_classic
+        else:
+            raise ValueError('Unknown flavour', flavour)
 
     @staticmethod
     def get_alias_from_url(url: URL) -> str | None:
@@ -890,33 +872,36 @@ class GithubResolver(BaseResolver):
             return '/'.join(url.parts[1:3])
 
     async def resolve_one(self, defn: Defn, metadata: None) -> models.Pkg:
-        from aiohttp import ClientResponseError
+        github_get = self.manager.web_client.get
+        github_token = self.manager.config.global_config.access_tokens.github
+        if github_token is not None:
+            github_get = partial(
+                github_get, headers={'Authorization': f'token {github_token.get_secret_value()}'}
+            )
 
         repo_url = self.repos_api_url / defn.alias
 
-        try:
-            project_metadata: _GithubRepo = await manager.cache_response(
-                self.manager, repo_url, {'hours': 1}
-            )
-        except ClientResponseError as error:
-            if error.status == 404:
+        async with github_get(repo_url, {'hours': 1}) as response:
+            if response.status == 404:
                 raise R.PkgNonexistent
-            raise
+            response.raise_for_status()
+            project_metadata: _GithubRepo = await response.json()
 
         if defn.strategy is Strategy.version:
             assert defn.version
             release_url = repo_url / 'releases/tags' / defn.version
         elif defn.strategy is Strategy.latest:
+            # Includes pre-releases
             release_url = (repo_url / 'releases').with_query(per_page='1')
         else:
-            # The latest release is the most recent non-prerelease,
-            # non-draft release, sorted by the ``created_at`` attribute.
-            # See: https://docs.github.com/en/free-pro-team@latest/rest/reference/repos#get-the-latest-release
+            # The latest release is the most recent release which is neither
+            # a pre-release nor a draft
             release_url = repo_url / 'releases/latest'
 
-        async with self.manager.web_client.get(release_url) as response:
+        async with github_get(release_url, {'minutes': 5}) as response:
             if response.status == 404:
                 raise R.PkgFileUnavailable('release not found')
+            response.raise_for_status()
 
             response_json = await response.json()
             if defn.strategy is Strategy.latest:
@@ -930,85 +915,94 @@ class GithubResolver(BaseResolver):
             None,
         )
         if release_json is None:
-            logger.info(f'no release.json found for {defn}; inspecting assets')
+            raise R.PkgFileUnavailable('no `release.json` attached to release')
 
-            def is_valid_asset(asset: _GithubRelease_Asset):
-                return (
-                    # There is something of a convention that Classic archives
-                    # end in '-classic' and lib-less archives end in '-nolib'.
-                    # Archives produced by packager follow this convention
-                    not asset['name'].endswith(('-nolib.zip', '-nolib-classic.zip'))
-                    and asset['content_type']
-                    in {'application/zip', 'application/x-zip-compressed'}
-                    # A failed upload has a state value of 'starter'
-                    and asset['state'] == 'uploaded'
-                )
+        async with self.manager.web_client.get(
+            release_json['browser_download_url'], {'days': 1}
+        ) as response:
+            response.raise_for_status()
+            packager_metadata: _PackagerReleaseJson = await response.json()
 
-            matching_asset = next(
-                chain(
-                    (
-                        a
-                        for a in assets
-                        if is_valid_asset(a)
-                        and a['name'].endswith('-classic.zip')
-                        is (self.manager.config.game_flavour is Flavour.vanilla_classic)
-                    ),
-                    filter(is_valid_asset, assets),
-                ),
-                None,
-            )
-            if matching_asset is None:
-                raise R.PkgFileUnavailable
+        releases = packager_metadata['releases']
+        if not releases:
+            raise R.PkgFileUnavailable('no files available for download')
 
-        else:
-            logger.info(f'reading metadata for {defn} from release.json')
-
-            packager_metadata: _PackagerReleaseJson = await manager.cache_response(
-                self.manager, release_json['browser_download_url'], {'days': 1}
-            )
-            releases = packager_metadata['releases']
-            if not releases:
-                raise R.PkgFileUnavailable('no files available for download')
-
-            matching_release = next(
-                (
-                    r
-                    for r in releases
-                    if r['nolib'] is False
-                    and any(
-                        m['flavor'] == self.release_json_flavours[self.manager.config.game_flavour]
-                        for m in r['metadata']
-                    )
-                ),
-                None,
-            )
-            if matching_release is None:
-                raise R.PkgFileUnavailable(f'no files matching {self.manager.config.game_flavour}')
-
-            matching_asset = next(
-                (
-                    a
-                    for a in assets
-                    if a['name'] == matching_release['filename'] and a['state'] == 'uploaded'
-                ),
-                None,
-            )
-            if matching_asset is None:
-                raise R.PkgFileUnavailable(f'{matching_release["filename"]} not found')
-
-        return models.Pkg(
-            source=self.source,
-            id=project_metadata['full_name'],
-            slug=project_metadata['full_name'].lower(),
-            name=project_metadata['name'],
-            description=project_metadata['description'] or '',
-            url=project_metadata['html_url'],
-            download_url=matching_asset['browser_download_url'],
-            date_published=release_metadata['published_at'],
-            version=release_metadata['tag_name'],
-            changelog_url=_format_data_changelog(release_metadata['body']),
-            options={'strategy': defn.strategy},
+        wanted_flavour = self._flavour_to_release_json_flavour(self.manager.config.game_flavour)
+        matching_release = next(
+            (
+                r
+                for r in releases
+                if r['nolib'] is False
+                and any(m['flavor'] == wanted_flavour for m in r['metadata'])
+            ),
+            None,
         )
+        if matching_release is None:
+            raise R.PkgFileUnavailable(f'no files matching {self.manager.config.game_flavour}')
+
+        matching_asset = next(
+            (
+                a
+                for a in assets
+                if a['name'] == matching_release['filename'] and a['state'] == 'uploaded'
+            ),
+            None,
+        )
+        if matching_asset is None:
+            raise R.PkgFileUnavailable(f'{matching_release["filename"]} not found')
+
+        return models.Pkg.parse_obj(
+            {
+                'source': self.source,
+                'id': project_metadata['full_name'],
+                'slug': project_metadata['full_name'].lower(),
+                'name': project_metadata['name'],
+                'description': project_metadata['description'] or '',
+                'url': project_metadata['html_url'],
+                'download_url': matching_asset['browser_download_url'],
+                'date_published': release_metadata['published_at'],
+                'version': release_metadata['tag_name'],
+                'changelog_url': _format_data_changelog(release_metadata['body']),
+                'options': {'strategy': defn.strategy},
+            }
+        )
+
+    @classmethod
+    async def catalogue(
+        cls, web_client: _deferred_types.aiohttp.ClientSession
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
+        import csv
+        import io
+
+        logger.debug(f'retrieving {cls.generated_catalogue_csv_url}')
+        async with web_client.get(cls.generated_catalogue_csv_url) as response:
+            catalogue_csv = await response.text()
+
+        for entry in csv.DictReader(io.StringIO(catalogue_csv)):
+            if entry['has_release_json'] != 'True':
+                continue
+
+            yield BaseCatatalogueEntry.parse_obj(
+                {
+                    'source': cls.source,
+                    'id': entry['full_name'],
+                    'slug': entry['full_name'].lower(),
+                    'name': entry['name'],
+                    'url': entry['url'],
+                    'game_flavours': {
+                        cls._release_json_flavour_to_flavour(f)
+                        for f in entry['flavors'].split(',')
+                    },
+                    'download_count': 1,
+                    'last_updated': datetime.fromisoformat(entry['last_updated']),
+                    'same_as': [
+                        {'source': i, 'id': v}
+                        for i in ('curse', 'wowi')
+                        for v in (entry[f'{i}_id'],)
+                        if v
+                    ],
+                }
+            )
 
 
 class InstawowResolver(BaseResolver):
@@ -1028,39 +1022,44 @@ class InstawowResolver(BaseResolver):
         except StopIteration:
             raise R.PkgNonexistent
 
-        from .wa_updater import BuilderConfig, WaCompanionBuilder
+        from .wa_updater import WaCompanionBuilder
 
-        builder = WaCompanionBuilder(self.manager, BuilderConfig())
+        builder = WaCompanionBuilder(self.manager)
         if source_id == '1':
             await builder.build()
 
-        return models.Pkg(
-            source=self.source,
-            id=source_id,
-            slug=slug,
-            name='WeakAuras Companion',
-            description='A WeakAuras Companion clone.',
-            url='https://github.com/layday/instawow',
-            download_url=builder.addon_zip_path.as_uri(),
-            date_published=datetime.now(timezone.utc),
-            version=(await builder.get_checksum())[:7],
-            changelog_url=builder.changelog_path.as_uri(),
-            options={'strategy': defn.strategy},
+        return models.Pkg.parse_obj(
+            {
+                'source': self.source,
+                'id': source_id,
+                'slug': slug,
+                'name': 'WeakAuras Companion',
+                'description': 'A WeakAuras Companion clone.',
+                'url': 'https://github.com/layday/instawow',
+                'download_url': builder.addon_zip_path.as_uri(),
+                'date_published': datetime.now(timezone.utc),
+                'version': (await builder.get_checksum())[:7],
+                'changelog_url': builder.changelog_path.as_uri(),
+                'options': {'strategy': defn.strategy},
+            }
         )
 
     @classmethod
     async def catalogue(
         cls, web_client: _deferred_types.aiohttp.ClientSession
-    ) -> AsyncIterator[CatatalogueBaseEntry]:
-        yield CatatalogueBaseEntry(
-            source=cls.source,
-            id='1',
-            slug='weakauras-companion-autoupdate',
-            name='WeakAuras Companion',
-            folders=[
-                ['WeakAurasCompanion'],
-            ],
-            game_flavours=set(Flavour),
-            download_count=1,
-            last_updated=datetime.now(timezone.utc),
+    ) -> AsyncIterator[BaseCatatalogueEntry]:
+        yield BaseCatatalogueEntry.parse_obj(
+            {
+                'source': cls.source,
+                'id': '1',
+                'slug': 'weakauras-companion-autoupdate',
+                'name': 'WeakAuras Companion',
+                'url': 'https://github.com/layday/instawow',
+                'game_flavours': set(Flavour),
+                'download_count': 1,
+                'last_updated': datetime.now(timezone.utc),
+                'folders': [
+                    {'WeakAurasCompanion'},
+                ],
+            }
         )
