@@ -6,7 +6,7 @@ from collections.abc import Awaitable, Callable, Iterable, Iterator, Mapping, Se
 from datetime import datetime, timedelta
 import enum
 from functools import partial, wraps
-from itertools import chain, repeat
+from itertools import chain, groupby, repeat
 import os
 from pathlib import Path, PurePath
 import posixpath
@@ -14,9 +14,14 @@ from shutil import move as _move
 from tempfile import mkdtemp
 from typing import Any, Generic, Hashable, TypeVar, overload
 
+import pydantic
+from typing_extensions import ParamSpec
+
 _T = TypeVar('_T')
 _U = TypeVar('_U')
-_H = TypeVar('_H', bound=Hashable)
+_THashable = TypeVar('_THashable', bound=Hashable)
+_TBaseModel = TypeVar('_TBaseModel', bound=pydantic.BaseModel)
+_P = ParamSpec('_P')
 
 
 class StrEnum(str, enum.Enum):
@@ -81,11 +86,17 @@ def chain_dict(
     return dict(chain(zip(keys, repeat(default)), *overrides))
 
 
-def uniq(it: Iterable[_H]) -> list[_H]:
+def uniq(it: Iterable[_THashable]) -> list[_THashable]:
     "Deduplicate hashable items in an iterable maintaining insertion order."
     return list(dict.fromkeys(it))
 
 
+def all_eq(it: Iterable[object]) -> bool:
+    "Check that all elements of an iterable are equal."
+    groups = groupby(it)
+    return next(groups, True) and not next(groups, False)
+
+
 def merge_intersecting_sets(it: Iterable[frozenset[_T]]) -> Iterator[frozenset[_T]]:
     "Recursively merge intersecting sets in a collection."
     many_sets = list(it)
@@ -130,7 +141,7 @@ def run_in_thread(
 
 
 @overload
-def run_in_thread(fn: Callable[..., _U]) -> Callable[..., Awaitable[_U]]:
+def run_in_thread(fn: Callable[_P, _U]) -> Callable[_P, Awaitable[_U]]:
     ...
 
 
@@ -234,7 +245,7 @@ def file_uri_to_path(file_uri: str) -> str:
     return unprefixed_path
 
 
-def normalise_names(replace_delim: str):
+def normalise_names(replace_delim: str) -> Callable[[str], str]:
     import string
 
     trans_table = str.maketrans(dict.fromkeys(string.punctuation, ' '))
@@ -243,3 +254,7 @@ def normalise_names(replace_delim: str):
         return replace_delim.join(value.casefold().translate(trans_table).split())
 
     return normalise
+
+
+def evolve_model_obj(model_obj: _TBaseModel, **kwargs: object) -> _TBaseModel:
+    return model_obj.__class__.parse_obj({**model_obj.__dict__, **kwargs})
