@@ -5,51 +5,44 @@ from functools import partial, reduce
 from itertools import chain, product
 import time
 import typing
-from typing import TYPE_CHECKING, Any, TypeVar
+from typing import Any, ClassVar, TypeVar
 
 from loguru import logger
 from pydantic import BaseModel, Field, validator
-from typing_extensions import Literal, Protocol, TypeAlias, TypedDict
+from typing_extensions import Literal
+from typing_extensions import NotRequired as N
+from typing_extensions import Self, TypeAlias, TypedDict
 from yarl import URL
 
-from . import manager
-from .config import BaseConfig, Flavour
+from .common import Flavour
+from .manager import Manager
 from .utils import bucketise, chain_dict, gather
 from .utils import run_in_thread as t
 from .utils import shasum
 
-# ``NotRequired`` is provisional and does not exist at runtime
-if TYPE_CHECKING:  # pragma: no cover
-    from typing_extensions import NotRequired as N
-
-
 _TAuras = TypeVar('_TAuras', bound='BaseAuras')
-_Slug = str
-_ImportString = str
+_ImportString = _Slug = str
 _AuraGroup: TypeAlias = 'Sequence[tuple[Sequence[WeakAura], WagoApiResponse, _ImportString]]'
 
 IMPORT_API_URL = URL('https://data.wago.io/api/raw/encoded')
 
 
-class BuilderConfig(BaseConfig):
-    wago_api_key: typing.Optional[str] = None
-
-
 class BaseAuras(
     BaseModel,
     arbitrary_types_allowed=True,
     json_encoders={URL: str},
 ):
-    class Meta(Protocol):
-        api_url: URL
-        filename: str
+    api_url: ClassVar[URL]
+    filename: ClassVar[str]
+
+    __root__: typing.Dict[_Slug, typing.List[Any]]
 
     @classmethod
-    def from_lua_table(cls: type[_TAuras], lua_table: Any) -> _TAuras:
+    def from_lua_table(cls, lua_table: Any) -> Self:
         raise NotImplementedError
 
 
-def _merge_auras(auras: Iterable[_TAuras]) -> dict[type, _TAuras]:
+def _merge_auras(auras: Iterable[_TAuras]) -> dict[type[_TAuras], _TAuras]:
     "Merge auras of the same type."
     return {
         t: t(__root__=reduce(lambda a, b: {**a, **b}, (i.__root__ for i in a)))
@@ -69,7 +62,7 @@ class WeakAura(
     version: int
 
     @classmethod
-    def from_lua_table(cls, lua_table: Any) -> WeakAura | None:
+    def from_lua_table(cls, lua_table: Any):
         url_string = lua_table.get('url')
         if url_string is not None:
             url = URL(url_string)
@@ -77,21 +70,20 @@ class WeakAura(
                 return cls.parse_obj({**lua_table, 'url': url})
 
     @validator('url', pre=True)
-    def _url_to_URL(cls, value: str | URL) -> URL:
+    def _url_to_URL(cls, value: str | URL):
         if not isinstance(value, URL):
             value = URL(value)
         return value
 
 
 class WeakAuras(BaseAuras):
-    class Meta:
-        api_url = URL('https://data.wago.io/api/check/weakauras')
-        filename = 'WeakAuras.lua'
+    api_url = URL('https://data.wago.io/api/check/weakauras')
+    filename = 'WeakAuras.lua'
 
     __root__: typing.Dict[_Slug, typing.List[WeakAura]]
 
     @classmethod
-    def from_lua_table(cls, lua_table: Any) -> WeakAuras:
+    def from_lua_table(cls, lua_table: Any):
         auras = (
             a for t in lua_table['displays'].values() for a in (WeakAura.from_lua_table(t),) if a
         )
@@ -105,14 +97,13 @@ class Plateroo(WeakAura):
 
 
 class Plateroos(BaseAuras):
-    class Meta:
-        api_url = URL('https://data.wago.io/api/check/plater')
-        filename = 'Plater.lua'
+    api_url = URL('https://data.wago.io/api/check/plater')
+    filename = 'Plater.lua'
 
     __root__: typing.Dict[_Slug, typing.List[Plateroo]]
 
     @classmethod
-    def from_lua_table(cls, lua_table: Any) -> Plateroos:
+    def from_lua_table(cls, lua_table: Any):
         auras = (
             a
             for n, p in lua_table['profiles'].items()
@@ -153,9 +144,10 @@ class WagoApiResponse_Changelog(TypedDict):
 class WaCompanionBuilder:
     """A WeakAuras Companion port for shellfolk."""
 
-    def __init__(self, manager: manager.Manager, builder_config: BuilderConfig) -> None:
+    def __init__(self, manager: Manager) -> None:
         self.manager = manager
-        self.builder_config = builder_config
+
+        self.access_token = self.manager.config.global_config.access_tokens.wago
 
         output_folder = self.manager.config.plugin_dir / __name__
         self.addon_zip_path = output_folder / 'WeakAurasCompanion.zip'
@@ -164,71 +156,75 @@ class WaCompanionBuilder:
 
     @staticmethod
     def extract_auras(model: type[_TAuras], source: str) -> _TAuras:
-        from ._custom_slpp import SLPP
+        from ._custom_slpp import loads
 
-        source_after_assignment = source[source.find('=') + 1 :]
-        lua_table = SLPP(source_after_assignment).decode()
-        return model.from_lua_table(lua_table)
+        lua_table = loads(f'{{{source}}}')
+        (aura_table,) = lua_table.values()
+        return model.from_lua_table(aura_table)
 
     def extract_installed_auras(self) -> Iterator[WeakAuras | Plateroos]:
         flavour_root = self.manager.config.addon_dir.parents[1]
-        saved_vars_of_every_account = flavour_root.glob('WTF/Account/*/SavedVariables')
+        saved_vars_by_account = flavour_root.glob('WTF/Account/*/SavedVariables')
         for saved_vars, model in product(
-            saved_vars_of_every_account,
+            saved_vars_by_account,
             [WeakAuras, Plateroos],
         ):
-            file = saved_vars / model.Meta.filename
+            file = saved_vars / model.filename
             if not file.exists():
                 logger.info(f'{file} not found')
             else:
                 content = file.read_text(encoding='utf-8-sig', errors='replace')
-                aura_group_cache = self.manager.config.cache_dir / shasum(content)
+                aura_group_cache = self.manager.config.global_config.cache_dir / shasum(content)
                 if aura_group_cache.exists():
                     logger.info(f'loading {file} from cache at {aura_group_cache}')
                     aura_groups = model.parse_file(aura_group_cache)
                 else:
                     start = time.perf_counter()
                     aura_groups = self.extract_auras(model, content)
-                    logger.debug(f'{model.__name__} extracted in {time.perf_counter() - start}s')
+                    logger.debug(
+                        f'extracted {model.__name__} in {time.perf_counter() - start:.3f}s'
+                    )
                     aura_group_cache.write_text(aura_groups.json(), encoding='utf-8')
                 yield aura_groups
 
-    async def _fetch_wago_metadata(
-        self, api_url: URL, aura_ids: Iterable[str]
-    ) -> list[WagoApiResponse]:
-        from aiohttp import ClientResponseError
+    async def _fetch_wago_metadata(self, api_url: URL, aura_ids: Iterable[str]):
+        async with self.manager.web_client.get(
+            api_url.with_query(ids=','.join(aura_ids)),
+            {'minutes': 30},
+            label='Fetching aura metadata',
+            headers={
+                'api-key': self.access_token.get_secret_value()
+                if self.access_token is not None
+                else ''
+            },
+        ) as response:
+            metadata: list[WagoApiResponse]
+            if response.status == 404:
+                metadata = []
+                return metadata
+            response.raise_for_status()
+            metadata = await response.json()
+            return sorted(metadata, key=lambda r: r['slug'])
 
-        try:
-            return sorted(
-                await manager.cache_response(
-                    self.manager,
-                    api_url.with_query(ids=','.join(aura_ids)),
-                    {'minutes': 30},
-                    label='Fetching aura metadata',
-                    request_extra={'headers': {'api-key': self.builder_config.wago_api_key or ''}},
-                ),
-                key=lambda r: r['slug'],
-            )
-        except ClientResponseError as error:
-            if error.status != 404:
-                raise
-            return []
-
-    async def _fetch_wago_import_string(self, aura: WagoApiResponse) -> str:
-        return await manager.cache_response(
-            self.manager,
+    async def _fetch_wago_import_string(self, aura: WagoApiResponse):
+        async with self.manager.web_client.get(
             IMPORT_API_URL.with_query(id=aura['_id']).with_fragment(str(aura['version'])),
             {'days': 30},
+            headers={
+                'api-key': self.access_token.get_secret_value()
+                if self.access_token is not None
+                else ''
+            },
             label=f"Fetching aura '{aura['slug']}'",
-            is_json=False,
-            request_extra={'headers': {'api-key': self.builder_config.wago_api_key or ''}},
-        )
+        ) as response:
+            response.raise_for_status()
+            return await response.text()
 
     async def get_remote_auras(self, auras: BaseAuras) -> _AuraGroup:
         if not auras.__root__:
             return []
 
-        metadata = await self._fetch_wago_metadata(auras.Meta.api_url, auras.__root__)
+        metadata = await self._fetch_wago_metadata(auras.api_url, auras.__root__)
         import_strings = await gather(self._fetch_wago_import_string(r) for r in metadata)
         return [(auras.__root__[r['slug']], r, i) for r, i in zip(metadata, import_strings)]
 
@@ -243,7 +239,7 @@ class WaCompanionBuilder:
     def _get_toc_number(self) -> str:
         game_flavour: Flavour = self.manager.config.game_flavour
         if game_flavour is Flavour.retail:
-            return '90100'
+            return '90105'
         elif game_flavour is Flavour.vanilla_classic:
             return '11400'
         elif game_flavour is Flavour.burning_crusade_classic:
