@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from collections.abc import Callable
 from functools import lru_cache
 from io import BytesIO
 import json
@@ -9,10 +10,13 @@ import re
 from typing import Any
 from zipfile import ZipFile
 
+import aiohttp
+from aresponses import ResponsesMockServer
 import pytest
 
 from instawow import __version__
-from instawow.config import Config, Flavour
+from instawow.common import Flavour
+from instawow.config import Config, GlobalConfig
 from instawow.manager import Manager, init_web_client
 
 inf = float('inf')
@@ -20,22 +24,19 @@ inf = float('inf')
 FIXTURES = Path(__file__).parent / 'fixtures'
 
 
-def pytest_addoption(parser):
-    parser.addoption('--iw-no-mock', action='store_true')
+def pytest_addoption(parser: pytest.Parser):
+    parser.addoption('--iw-no-mock-http', action='store_true')
 
 
-def should_mock(fn):
+def should_mock(fn: Callable[..., object]):
     import inspect
     import warnings
 
-    def wrapper(request):
-        if request.config.getoption('--iw-no-mock'):
+    def wrapper(request: pytest.FixtureRequest):
+        if request.config.getoption('--iw-no-mock-http'):
             warnings.warn('not mocking')
             return None
-        elif any(m.name == 'iw_no_mock' for m in request.node.iter_markers()):
-            return None
-        elif request.module.__name__ == 'test_json_rpc_api':
-            # aresponses conflicts with aiohttp's own test server
+        elif any(m.name == 'iw_no_mock_http' for m in request.node.iter_markers()):
             return None
 
         args = (request.getfixturevalue(p) for p in inspect.signature(fn).parameters)
@@ -44,10 +45,12 @@ def should_mock(fn):
     return wrapper
 
 
+@lru_cache(maxsize=None)
 def load_fixture(filename: str):
     return (FIXTURES / filename).read_bytes()
 
 
+@lru_cache(maxsize=None)
 def load_json_fixture(filename: str):
     return json.loads(load_fixture(filename))
 
@@ -64,31 +67,27 @@ def make_addon_zip(*folders: str):
 
 @pytest.fixture(scope='session', autouse=True)
 def iw_temp_dir(tmp_path_factory: pytest.TempPathFactory):
-    temp_dir = tmp_path_factory.mktemp('temp')
+    temp_dir = tmp_path_factory.mktemp('temp') / 'instawow'
     os.environ['INSTAWOW_TEMP_DIR'] = str(temp_dir)
-    yield temp_dir
+    return temp_dir
+
+
+@pytest.fixture
+def iw_global_config_values(tmp_path: Path, iw_temp_dir: Path):
+    return {'temp_dir': iw_temp_dir, 'config_dir': tmp_path / 'config'}
 
 
 @pytest.fixture(params=Flavour)
-def iw_config_dict_no_config_dir(tmp_path: Path, request, iw_temp_dir: Path):
+def iw_config_values(request: Any, tmp_path: Path):
     addons = tmp_path / 'wow' / 'interface' / 'addons'
     addons.mkdir(parents=True)
-    return {
-        'profile': '__default__',
-        'addon_dir': addons,
-        'temp_dir': iw_temp_dir,
-        'game_flavour': request.param,
-    }
+    return {'profile': '__default__', 'addon_dir': addons, 'game_flavour': request.param}
 
 
 @pytest.fixture
-def iw_config_dict(tmp_path: Path, iw_config_dict_no_config_dir: dict[str, Any]):
-    return {**iw_config_dict_no_config_dir, 'config_dir': tmp_path / 'config'}
-
-
-@pytest.fixture
-def iw_config(iw_config_dict: dict[str, Any]):
-    yield Config(**iw_config_dict).write()
+def iw_config(iw_config_values: dict[str, Any], iw_global_config_values: dict[str, Any]):
+    global_config = GlobalConfig(**iw_global_config_values).write()
+    return Config(global_config=global_config, **iw_config_values).write()
 
 
 @pytest.fixture
@@ -98,15 +97,16 @@ async def iw_web_client():
 
 
 @pytest.fixture
-def iw_manager(iw_config: Config, iw_web_client):
-    manager = Manager.from_config(iw_config)
-    manager.contextualise(web_client=iw_web_client)
+def iw_manager(iw_config: Config, iw_web_client: aiohttp.ClientSession):
+    Manager.contextualise(web_client=iw_web_client)
+    manager, close_db_conn = Manager.from_config(iw_config)
     yield manager
+    close_db_conn()
 
 
-@pytest.fixture
+@pytest.fixture(autouse=True)
 @should_mock
-def mock_pypi(aresponses):
+def mock_aiohttp_requests(aresponses: ResponsesMockServer):
     aresponses.add(
         'pypi.org',
         '/pypi/instawow/json',
@@ -115,22 +115,14 @@ def mock_pypi(aresponses):
         repeat=inf,
     )
 
-
-@pytest.fixture
-@should_mock
-def mock_master_catalogue(aresponses):
     aresponses.add(
         'raw.githubusercontent.com',
-        aresponses.ANY,
+        '/layday/instawow-data/data/base-catalogue-v5.compact.json',
         'get',
-        load_json_fixture('master-catalogue.json'),
+        load_json_fixture('base-catalogue-v5.compact.json'),
         repeat=inf,
     )
 
-
-@pytest.fixture
-@should_mock
-def mock_curse(aresponses, mock_master_catalogue):
     aresponses.add(
         'addons-ecs.forgesvc.net',
         '/api/v2/addon',
@@ -160,10 +152,6 @@ def mock_curse(aresponses, mock_master_catalogue):
         repeat=inf,
     )
 
-
-@pytest.fixture
-@should_mock
-def mock_wowi(aresponses, mock_master_catalogue):
     aresponses.add(
         'api.mmoui.com',
         '/v3/game/WOW/filelist.json',
@@ -186,10 +174,6 @@ def mock_wowi(aresponses, mock_master_catalogue):
         repeat=inf,
     )
 
-
-@pytest.fixture
-@should_mock
-def mock_tukui(aresponses, mock_master_catalogue):
     aresponses.add(
         'www.tukui.org',
         '/api.php?ui=tukui',
@@ -261,10 +245,6 @@ def mock_tukui(aresponses, mock_master_catalogue):
         repeat=inf,
     )
 
-
-@pytest.fixture
-@should_mock
-def mock_github(aresponses):
     aresponses.add(
         'api.github.com',
         '/repos/nebularg/PackagerTest',
@@ -281,52 +261,33 @@ def mock_github(aresponses):
     )
     aresponses.add(
         'github.com',
-        '/nebularg/PackagerTest/releases/download/v1.9.6/release.json',
+        '/nebularg/PackagerTest/releases/download/v1.9.7/release.json',
         'get',
         load_json_fixture('github-release-release-json-release-json.json'),
         repeat=inf,
     )
     aresponses.add(
         'api.github.com',
-        '/repos/AdiAddons/AdiButtonAuras',
+        re.compile(r'^/repos/p3lim-wow/Molinari$', re.IGNORECASE),
         'get',
-        load_json_fixture('github-repo-legacy-lib-and-nolib.json'),
+        load_json_fixture('github-repo-molinari.json'),
         repeat=inf,
     )
     aresponses.add(
         'api.github.com',
-        '/repos/AdiAddons/AdiButtonAuras/releases/latest',
+        re.compile(r'^/repos/p3lim-wow/Molinari/releases/latest$', re.IGNORECASE),
         'get',
-        load_json_fixture('github-release-legacy-lib-and-nolib.json'),
+        load_json_fixture('github-release-molinari.json'),
         repeat=inf,
     )
     aresponses.add(
-        'api.github.com',
-        '/repos/AdiAddons/AdiButtonAuras/releases?per_page=1',
+        'github.com',
+        re.compile(
+            r'^/p3lim-wow/Molinari/releases/download/90105\.81-Release/release\.json$',
+            re.IGNORECASE,
+        ),
         'get',
-        [load_json_fixture('github-release-legacy-lib-and-nolib.json')],
-        match_querystring=True,
-        repeat=inf,
-    )
-    aresponses.add(
-        'api.github.com',
-        '/repos/AdiAddons/AdiButtonAuras/releases/tags/2.1.0',
-        'get',
-        load_json_fixture('github-release-legacy-lib-and-nolib-older-version.json'),
-        repeat=inf,
-    )
-    aresponses.add(
-        'api.github.com',
-        '/repos/p3lim-wow/Molinari',
-        'get',
-        load_json_fixture('github-repo-legacy-retail-and-classic.json'),
-        repeat=inf,
-    )
-    aresponses.add(
-        'api.github.com',
-        '/repos/p3lim-wow/Molinari/releases/latest',
-        'get',
-        load_json_fixture('github-release-legacy-retail-and-classic.json'),
+        load_json_fixture('github-release-molinari-release-json.json'),
         repeat=inf,
     )
     aresponses.add(
@@ -365,15 +326,17 @@ def mock_github(aresponses):
         repeat=inf,
     )
 
-
-@pytest.fixture(autouse=True)
-@should_mock
-def mock_all(
-    mock_pypi,
-    mock_master_catalogue,
-    mock_curse,
-    mock_wowi,
-    mock_tukui,
-    mock_github,
-):
-    pass
+    aresponses.add(
+        'github.com',
+        '/login/device/code',
+        'post',
+        load_json_fixture('github-oauth-login-device-code.json'),
+        repeat=inf,
+    )
+    aresponses.add(
+        'github.com',
+        '/login/oauth/access_token',
+        'post',
+        load_json_fixture('github-oauth-login-access-token.json'),
+        repeat=inf,
+    )
