@@ -11,9 +11,10 @@ echo '[2382,20338,23350,288981,322865,2398,326516,333072,402180]' \
 http get https://addons-ecs.forgesvc.net/api/v2/addon/20338/files -b \
     | jq -r \
     > "$DIR"/curse-addon-files.json
-http get https://addons-ecs.forgesvc.net/api/v2/addon/20338/file/3475338/changelog -b \
+http get https://addons-ecs.forgesvc.net/api/v2/addon/20338/file/3576747/changelog -b \
     > "$DIR"/curse-addon-changelog.txt
 
+
 http get https://api.mmoui.com/v3/game/WOW/filelist.json -b \
     | jq -r '.[] | select(.UID == "13188") | [.]' \
     > "$DIR"/wowi-filelist.json
@@ -21,6 +22,7 @@ http get https://api.mmoui.com/v3/game/WOW/filedetails/13188.json -b \
     | jq -r \
     > "$DIR"/wowi-filedetails.json
 
+
 http get 'https://www.tukui.org/api.php?ui=tukui' -b \
     | jq -r \
     > "$DIR"/tukui-ui--tukui.json
@@ -44,24 +46,18 @@ http get 'https://api.github.com/repos/nebularg/PackagerTest' -b \
 http get 'https://api.github.com/repos/nebularg/PackagerTest/releases/latest' -b \
     | jq -r \
     > "$DIR"/github-release-release-json.json
-http --follow get 'https://github.com/nebularg/PackagerTest/releases/download/v1.9.6/release.json' -b \
+http --follow get 'https://github.com/nebularg/PackagerTest/releases/download/v1.9.7/release.json' -b \
     | jq -r \
     > "$DIR"/github-release-release-json-release-json.json
-http get 'https://api.github.com/repos/AdiAddons/AdiButtonAuras' -b \
-    | jq -r \
-    > "$DIR"/github-repo-legacy-lib-and-nolib.json
-http get 'https://api.github.com/repos/AdiAddons/AdiButtonAuras/releases/latest' -b \
-    | jq -r \
-    > "$DIR"/github-release-legacy-lib-and-nolib.json
-http get 'https://api.github.com/repos/AdiAddons/AdiButtonAuras/releases/tags/2.1.0' -b \
-    | jq -r \
-    > "$DIR"/github-release-legacy-lib-and-nolib-older-version.json
 http get 'https://api.github.com/repos/p3lim-wow/Molinari' -b \
     | jq -r \
-    > "$DIR"/github-repo-legacy-retail-and-classic.json
+    > "$DIR"/github-repo-molinari.json
 http get 'https://api.github.com/repos/p3lim-wow/Molinari/releases/latest' -b \
     | jq -r \
-    > "$DIR"/github-release-legacy-retail-and-classic.json
+    > "$DIR"/github-release-molinari.json
+http --follow get 'https://github.com/p3lim-wow/Molinari/releases/download/90105.81-Release/release.json' -b \
+    | jq -r \
+    > "$DIR"/github-release-molinari-release-json.json
 http get 'https://api.github.com/repos/AdiAddons/AdiBags' -b \
     | jq -r \
     > "$DIR"/github-repo-no-releases.json
@@ -69,6 +65,25 @@ http get 'https://api.github.com/repos/AdiAddons/AdiButtonAuras/releases/tags/2.
     | jq -r \
     > "$DIR"/github-release-no-assets.json
 
-http get https://raw.githubusercontent.com/layday/instawow-data/data/master-catalogue-v4.json -b \
+
+http get https://raw.githubusercontent.com/layday/instawow-data/data/base-catalogue-v5.compact.json -b \
     | jq -r \
-    > "$DIR"/master-catalogue.json
+    > "$DIR"/base-catalogue-v5.compact.json
+
+
+echo '{
+  "device_code": "3584d83530557fdd1f46af8289938c8ef79f9dc5",
+  "user_code": "WDJB-MJHT",
+  "verification_uri": "https://github.com/login/device",
+  "expires_in": 900,
+  "interval": 5
+}' \
+    | jq -r \
+    > "$DIR"/github-oauth-login-device-code.json
+echo '{
+  "access_token": "gho_16C7e42F292c6912E7710c838347Ae178B4a",
+  "token_type": "bearer",
+  "scope": "repo,gist"
+}' \
+    | jq -r \
+    > "$DIR"/github-oauth-login-access-token.json
