@@ -1,12 +1,14 @@
 from __future__ import annotations
 
+from collections.abc import Callable as C
 from functools import partial
 import json
 from typing import Any
 
-from aiohttp import ClientWebSocketResponse
+from aiohttp import ClientWebSocketResponse, WSServerHandshakeError
 from aiohttp.test_utils import TestClient, TestServer
 import pytest
+from yarl import URL
 
 from instawow.config import Config
 
@@ -14,46 +16,90 @@ try:
     from instawow_gui import json_rpc_server
 except ImportError:
     pytestmark = pytest.mark.skip(reason='instawow_gui is not available')
+else:
+    pytestmark = pytest.mark.iw_no_mock_http
 
 
 dumps = partial(json.dumps, default=str)
 
 
 @pytest.fixture
-async def ws(iw_config_dict: dict[str, Any], monkeypatch: pytest.MonkeyPatch):
-    monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_config_dict['config_dir']))
+async def ws_client(
+    monkeypatch: pytest.MonkeyPatch,
+    iw_global_config_values: dict[str, Any],
+):
+    monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_global_config_values['config_dir']))
     app = await json_rpc_server.create_app()
     server = TestServer(app)
-    async with TestClient(server) as client, client.ws_connect('/api') as ws:
+    async with TestClient(server) as client:
+        yield client
+
+
+@pytest.fixture
+async def ws(ws_client: TestClient):
+    async with ws_client.ws_connect('/api', origin=str(ws_client.make_url(''))) as ws:
         yield ws
 
 
 @pytest.mark.asyncio
-async def test_write_config(
-    request: pytest.FixtureRequest, iw_config_dict: dict[str, Any], ws: ClientWebSocketResponse
+async def test_no_origin_api_request_rejected(ws_client: TestClient):
+    with pytest.raises(WSServerHandshakeError):
+        async with ws_client.ws_connect('/api'):
+            pass
+
+
+@pytest.mark.parametrize(
+    'transform',
+    [
+        lambda u: u.with_scheme('ftp'),
+        lambda u: u.with_host('example.com'),
+        lambda u: u.with_port(21),
+    ],
+)
+@pytest.mark.asyncio
+async def test_disparate_origin_api_request_rejected(
+    ws_client: TestClient,
+    transform: C[[URL], URL],
 ):
-    config_values = {**iw_config_dict, 'profile': request.node.name}
+    with pytest.raises(WSServerHandshakeError):
+        async with ws_client.ws_connect('/api', origin=str(transform(ws_client.make_url('')))):
+            pass
+
+
+@pytest.mark.asyncio
+async def test_write_config(
+    request: pytest.FixtureRequest,
+    iw_global_config_values: dict[str, Any],
+    iw_config_values: dict[str, Any],
+    ws: ClientWebSocketResponse,
+):
+    config_values = {**iw_config_values, 'profile': request.node.name}
     rpc_request = {
         'jsonrpc': '2.0',
         'method': 'config/write',
-        'params': {'values': config_values, 'infer_game_flavour': False},
+        'params': {**config_values, 'infer_game_flavour': False},
         'id': request.node.name,
     }
     await ws.send_json(rpc_request, dumps=dumps)
     rpc_response = await ws.receive_json()
     assert rpc_response['id'] == request.node.name
-    assert Config.parse_obj(rpc_response['result']) == Config.parse_obj(config_values)
+    assert Config.parse_obj(rpc_response['result']) == Config.parse_obj(
+        {'global_config': iw_global_config_values, **config_values}
+    )
 
 
 @pytest.mark.asyncio
 async def test_write_config_with_invalid_params(
-    request: pytest.FixtureRequest, iw_config_dict: dict[str, Any], ws: ClientWebSocketResponse
+    request: pytest.FixtureRequest,
+    iw_config_values: dict[str, Any],
+    ws: ClientWebSocketResponse,
 ):
     rpc_request = {
         'jsonrpc': '2.0',
         'method': 'config/write',
         'params': {
-            'values': {**iw_config_dict, 'game_flavour': 'strawberry'},
+            **iw_config_values,
+            'game_flavour': 'strawberry',
             'infer_game_flavour': False,
         },
         'id': request.node.name,
@@ -62,8 +108,8 @@ async def test_write_config_with_invalid_params(
     rpc_response = await ws.receive_json()
     assert rpc_response['id'] == request.node.name
     assert rpc_response['error']
-    assert rpc_response['error']['code'] == -32001
-    assert rpc_response['error']['message'] == 'invalid configuration parameters'
+    assert rpc_response['error']['code'] == -32602
+    assert rpc_response['error']['message'] == 'Invalid method parameter(s).'
     assert rpc_response['error']['data'] == [
         {
             'loc': ['game_flavour'],
@@ -76,7 +122,8 @@ async def test_write_config_with_invalid_params(
 
 @pytest.mark.asyncio
 async def test_install_with_invalid_params(
-    request: pytest.FixtureRequest, ws: ClientWebSocketResponse
+    request: pytest.FixtureRequest,
+    ws: ClientWebSocketResponse,
 ):
     rpc_request = {
         'jsonrpc': '2.0',
@@ -92,7 +139,8 @@ async def test_install_with_invalid_params(
 @pytest.mark.xfail
 @pytest.mark.asyncio
 async def test_install_with_uninitialised_profile(
-    request: pytest.FixtureRequest, ws: ClientWebSocketResponse
+    request: pytest.FixtureRequest,
+    ws: ClientWebSocketResponse,
 ):
     rpc_request = {
         'jsonrpc': '2.0',
