@@ -4,11 +4,11 @@ import datetime
 from pathlib import Path
 
 from aiohttp import ClientError
+from aresponses import ResponsesMockServer
 import pytest
 
 from instawow import results as R
-from instawow.common import Strategy
-from instawow.config import Flavour
+from instawow.common import Flavour, Strategy
 from instawow.manager import Manager, is_outdated
 from instawow.models import Pkg
 from instawow.resolvers import Defn
@@ -196,7 +196,7 @@ async def test_removing_pkg_with_missing_folders(iw_manager: Manager, keep_folde
 async def test_basic_search(iw_manager: Manager):
     limit = 5
     results = await iw_manager.search('molinari', limit=limit)
-    assert len(results) == limit
+    assert len(results) <= 5
     assert {('curse', 'molinari'), ('wowi', '13188')} <= {
         (e.source, e.slug or e.id) for e in results
     }
@@ -225,10 +225,6 @@ async def test_search_source_filtering(iw_manager: Manager):
 @pytest.mark.asyncio
 async def test_search_date_filtering(iw_manager: Manager):
     start_date = datetime.datetime.now(tz=datetime.timezone.utc) - datetime.timedelta(days=365)
-
-    results = await iw_manager.search('molinari', limit=5)
-    assert any(e.last_updated < start_date for e in results)
-
     results = await iw_manager.search('molinari', limit=5, start_date=start_date)
     assert all(e.last_updated > start_date for e in results)
 
@@ -251,7 +247,7 @@ async def test_get_changelog_from_url_encoded_data_url(iw_manager: Manager):
 
 @pytest.mark.asyncio
 async def test_get_malformed_changelog(iw_manager: Manager):
-    with pytest.raises(ValueError, match='Unsupported URL with scheme'):
+    with pytest.raises(ValueError, match='Unsupported URI with scheme'):
         await iw_manager.get_changelog('')
 
 
@@ -261,7 +257,7 @@ async def test_get_changelog_from_file_uri(iw_manager: Manager):
         await iw_manager.get_changelog(
             (Path(__file__).parent / 'fixtures' / 'curse-addon-changelog.txt').as_uri()
         )
-    ).startswith('<h3>Changes in 90100.79-Release:</h3>')
+    ).startswith('<h3>Changes in 90105.81-Release:</h3>')
 
 
 @pytest.mark.asyncio
@@ -270,13 +266,13 @@ async def test_get_changelog_from_web_url(iw_manager: Manager):
         await iw_manager.get_changelog(
             'https://addons-ecs.forgesvc.net/api/v2/addon/20338/file/3475338/changelog'
         )
-    ).startswith('<h3>Changes in 90100.79-Release:</h3>')
+    ).startswith('<h3>Changes in 90105.81-Release:</h3>')
 
 
-@pytest.mark.iw_no_mock
+@pytest.mark.iw_no_mock_http
 @pytest.mark.asyncio
 async def test_is_outdated_works_in_variety_of_scenarios(
-    monkeypatch: pytest.MonkeyPatch, aresponses, iw_temp_dir: Path
+    monkeypatch: pytest.MonkeyPatch, aresponses: ResponsesMockServer, iw_temp_dir: Path
 ):
     pypi_version = iw_temp_dir.joinpath('.pypi_version')
     if pypi_version.exists():
