@@ -5,12 +5,12 @@ from pathlib import Path
 
 import pytest
 
-from instawow.config import Flavour
+from instawow.common import Flavour
 from instawow.manager import Manager
 from instawow.matchers import (
     AddonFolder,
     FolderAndDefnPairs,
-    get_unreconciled_folder_set,
+    get_unreconciled_folders,
     match_addon_names_with_folder_names,
     match_folder_name_subsets,
     match_toc_source_ids,
@@ -25,12 +25,6 @@ def write_addons(iw_manager: Manager, *addons: str):
         (iw_manager.config.addon_dir / addon / f'{addon}.toc').touch()
 
 
-@pytest.fixture
-def invalid_addons(iw_manager: Manager):
-    (iw_manager.config.addon_dir / 'foo').mkdir()
-    (iw_manager.config.addon_dir / 'bar').touch()
-
-
 @pytest.fixture
 def molinari(iw_manager: Manager):
     molinari_folder = iw_manager.config.addon_dir / 'Molinari'
@@ -55,8 +49,10 @@ def test_reconcile_addon_folder_can_extract_defns_from_toc(molinari: Path):
 
 
 @pytest.mark.asyncio
-async def test_reconcile_invalid_addons_discarded(iw_manager: Manager, invalid_addons: None):
-    folders = get_unreconciled_folder_set(iw_manager)
+async def test_reconcile_invalid_addons_discarded(iw_manager: Manager):
+    iw_manager.config.addon_dir.joinpath('foo').mkdir()
+    iw_manager.config.addon_dir.joinpath('bar').touch()
+    folders = get_unreconciled_folders(iw_manager)
     assert folders == frozenset()
     assert await match_toc_source_ids(iw_manager, folders) == []
     assert await match_folder_name_subsets(iw_manager, folders) == []
@@ -65,22 +61,25 @@ async def test_reconcile_invalid_addons_discarded(iw_manager: Manager, invalid_a
 @pytest.mark.asyncio
 @pytest.mark.parametrize(
     'test_func',
-    [match_toc_source_ids, match_addon_names_with_folder_names, match_folder_name_subsets],
+    [match_toc_source_ids, match_folder_name_subsets, match_addon_names_with_folder_names],
 )
 async def test_reconcile_multiple_defns_per_addon_contained_in_results(
     iw_manager: Manager,
     molinari: Path,
     test_func: Callable[[Manager, frozenset[AddonFolder]], Awaitable[FolderAndDefnPairs]],
 ):
-    ((_, matches),) = await test_func(iw_manager, get_unreconciled_folder_set(iw_manager))
-    assert {Defn('curse', '20338'), Defn('wowi', '13188')} == set(matches)
+    ((_, matches),) = await test_func(iw_manager, get_unreconciled_folders(iw_manager))
+    expected = {Defn('curse', '20338'), Defn('wowi', '13188')}
+    if test_func is match_addon_names_with_folder_names:
+        expected.add(Defn('github', 'p3lim-wow/Molinari'))
+    assert expected == set(matches)
 
 
 @pytest.mark.asyncio
 async def test_reconcile_results_vary_by_game_flavour(iw_manager: Manager):
     write_addons(iw_manager, 'AdiBags', 'AdiBags_Config')
     ((_, matches),) = await match_folder_name_subsets(
-        iw_manager, get_unreconciled_folder_set(iw_manager)
+        iw_manager, get_unreconciled_folders(iw_manager)
     )
     if iw_manager.config.game_flavour is Flavour.retail:
         assert {
