@@ -6,7 +6,7 @@ from yarl import URL
 from instawow.manager import Manager
 from instawow.models import Pkg
 from instawow.resolvers import Defn
-from instawow.wa_updater import BuilderConfig, WaCompanionBuilder, WeakAura, WeakAuras
+from instawow.wa_updater import WaCompanionBuilder, WeakAura, WeakAuras
 
 
 @pytest.fixture
@@ -15,7 +15,7 @@ def wa_saved_vars(iw_manager: Manager):
         iw_manager.config.addon_dir.parents[1] / 'WTF' / 'Account' / 'test' / 'SavedVariables'
     )
     saved_vars.mkdir(parents=True)
-    (saved_vars / WeakAuras.Meta.filename).write_text(
+    (saved_vars / WeakAuras.filename).write_text(
         '''\
 WeakAurasSaved = {
     ["displays"] = {
@@ -30,7 +30,7 @@ WeakAurasSaved = {
 
 @pytest.fixture
 def builder(iw_manager: Manager):
-    yield WaCompanionBuilder(iw_manager, BuilderConfig())
+    yield WaCompanionBuilder(iw_manager)
 
 
 def test_can_parse_empty_displays_table(builder: WaCompanionBuilder):
